@@ -22,14 +22,13 @@
  *
  */
 
-/**@CFILE nua.c Sofia-SIP User Agent Library API Implementation.
+/**@file nua.c High-Level User Agent Library - "nua" Implementation.
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
  * @author Kai Vehmanen <Kai.Vehmanen@nokia.com>
  * @author Pasi Rinne-Rahkola
  *
  * @date Created: Wed Feb 14 18:32:58 2001 ppessi
- *
  */
 
 #include "config.h"
@@ -64,14 +63,12 @@
 /* From AM_INIT/AC_INIT in our "config.h" */
 char const nua_version[] = VERSION;
 
-/**@var NUA_DEBUG
- *
- * Environment variable determining the debug log level for @b nua module.
+/**Environment variable determining the debug log level for @nua module.
  *
  * The NUA_DEBUG environment variable is used to determine the debug logging
- * level for @b nua module. The default level is 3.
+ * level for @nua module. The default level is 3.
  * 
- * @sa <su_debug.h>, nua_log, SOFIA_DEBUG
+ * @sa <sofia-sip/su_debug.h>, nua_log, SOFIA_DEBUG
  */
 extern char const NUA_DEBUG[];
 
@@ -79,16 +76,16 @@ extern char const NUA_DEBUG[];
 #define SU_DEBUG 3
 #endif
 
-/**Debug log for @b nua module. 
+/**Debug log for @nua module. 
  * 
- * The nua_log is the log object used by @b nua module. The level of
+ * The nua_log is the log object used by @nua module. The level of
  * #nua_log is set using #NUA_DEBUG environment variable.
  */
 su_log_t nua_log[] = { SU_LOG_INIT("nua", "NUA_DEBUG", SU_DEBUG) };
 
-/**Create a NUA agent.
+/**Create a @nua agent.
  *
- * This function creates a Sofia-SIP User Agent stack object (nua) and
+ * This function creates a Sofia-SIP User Agent stack object (@nua) and
  * initializes its parameters by given tagged values.
  *
  * @param root            Pointer to a root object
@@ -96,26 +93,32 @@ su_log_t nua_log[] = { SU_LOG_INIT("nua", "NUA_DEBUG", SU_DEBUG) };
  * @param magic           Pointer to callback context
  * @param tag,value,... List of tagged parameters
  *
- * @retval !=NULL a pointer to a NUA stack object \n
+ * @retval !=NULL a pointer to a @nua stack object \n
  * @retval NULL upon an error
  *
  * @par Related tags:
- *     NUTAG_MEDIA_ENABLE()     \n
- *     NUTAG_SOA_NAME()         \n
  *     NUTAG_PROXY()            \n
  *     NUTAG_URL()              \n
  *     NUTAG_SIPS_URL()         \n
  *     NUTAG_SIP_PARSER()       \n
  *     NUTAG_UICC()             \n
  *     NUTAG_CERTIFICATE_DIR()  \n
- *     all relevant NTATAG_* are passed to NTA 
+ *     and all tags listed in nua_set_params(), \n
+ *     and all relevant NTATAG_* are passed to NTA.
  *
  * @note
- * Both the NUTAG_URL and NUTAG_SIPS_URL() are used to pass arguments to
+ * From the @VERSION_1_12_2 all the nua_set_params() tags are processed. 
+ * Previously all nutags except NUTAG_SOA_NAME() and NUTAG_MEDIA_ENABLE()
+ * were ignored.
+ *
+ * @note
+ * Both the NUTAG_URL() and NUTAG_SIPS_URL() are used to pass arguments to
  * nta_agent_add_tport(). 
  *
  * @par Events:
  *     none
+ *
+ * @sa nua_shutdown(), nua_destroy(), nua_handle()
  */
 nua_t *nua_create(su_root_t *root,
 		  nua_callback_f callback,
@@ -140,7 +143,7 @@ nua_t *nua_create(su_root_t *root,
 
     ta_start(ta, tag, value);
 
-    nua->nua_args = ta_args(ta);
+    nua->nua_args = tl_adup(nua->nua_home, ta_args(ta));
 
     su_task_copy(nua->nua_client, su_root_task(root));
 
@@ -171,23 +174,8 @@ nua_t *nua_create(su_root_t *root,
   return nua;
 }
 
-/**Shutdown a NUA stack.
- *
- * Ongoing calls are released, registrations unregistered, and 
- * subscriptions terminated.  If the stack cannot terminate within 
- * 30 seconds, it sends the nua_r_shutdown event with status 500.
- *
- * @param nua         Pointer to NUA stack object
- *
- * @return
- *     nothing
- *
- * @par Related tags:
- *     none
- *
- * @par Events:
- *     #nua_r_shutdown
- */
+/* nua_shutdown() is documented with nua_stack_shutdown() */
+
 void nua_shutdown(nua_t *nua)
 {
   enter;
@@ -197,14 +185,14 @@ void nua_shutdown(nua_t *nua)
   nua_signal(nua, NULL, NULL, 1, nua_r_shutdown, 0, NULL, TAG_END());
 }
 
-/** Destroy the NUA stack.
+/** Destroy the @nua stack.
  *
  * Before calling nua_destroy() the application 
  * should call nua_shutdown and wait for successful #nua_r_shutdown event.
- * Shuts down and destroys the NUA stack. Ongoing calls, registrations, 
+ * Shuts down and destroys the @nua stack. Ongoing calls, registrations, 
  * and subscriptions are left as they are.
  *
- * @param nua         Pointer to NUA stack object
+ * @param nua         Pointer to @nua stack object
  *
  * @return
  *     nothing
@@ -214,6 +202,8 @@ void nua_shutdown(nua_t *nua)
  *
  * @par Events:
  *     none
+ *
+ * @sa nua_shutdown(), nua_create(), nua_handle_destroy(), nua_handle_unref()
  */
 void nua_destroy(nua_t *nua)
 {
@@ -237,14 +227,27 @@ void nua_destroy(nua_t *nua)
   }
 }
 
-/** Obtain default operation handle of the NUA stack object.
+/** Fetch callback context from nua.
+ *
+ * @param nua         Pointer to @nua stack object
+ *
+ * @return Callback context pointer.
+ *
+ * @NEW_1_12_4.
+ */
+nua_magic_t *nua_magic(nua_t *nua)
+{
+  return nua ? nua->nua_magic : NULL;
+}
+
+/** Obtain default operation handle of the @nua stack object.
  *
  * A default operation can be used for operations where the 
  * ultimate result is not important or can be discarded.
  *
- * @param nua         Pointer to NUA stack object
+ * @param nua         Pointer to @nua stack object
  *
- * @retval !=NULL Pointer to NUA operation handle
+ * @retval !=NULL Pointer to @nua operation handle
  * @retval NULL   No default operation exists
  *
  * @par Related tags:
@@ -263,7 +266,7 @@ nua_handle_t *nua_default(nua_t *nua)
  *
  * Allocates a new operation handle and associated storage.
  *
- * @param nua         Pointer to NUA stack object
+ * @param nua         Pointer to @nua stack object
  * @param hmagic      Pointer to callback context
  * @param tag, value, ... List of tagged parameters
  *
@@ -271,12 +274,22 @@ nua_handle_t *nua_default(nua_t *nua)
  * @retval NULL    Creation failed
  *
  * @par Related tags:
- *     Creates a copy of provided tags and they will 
- *     be used with every operation.
+ *     Duplicates the provided tags for use with every operation. Note that
+ *     NUTAG_URL() is converted to SIPTAG_TO() if there is no SIPTAG_TO(). 
+ *     And also vice versa, request-URI is taken from SIPTAG_TO() if there
+ *     is no NUTAG_URL(). Note that certain SIP headers cannot be saved with
+ *     the handle. They include @ContentLength, @CSeq, @RSeq, @RAck, and
+ *     @Timestamp.
+ *
+ * @par
+ *     nua_handle() accepts all the tags accepted by nua_set_hparams(), too.
+ *
  *
  * @par Events:
  *     none
  *
+ * @sa nua_handle_bind(), nua_handle_destroy(), nua_handle_ref(),
+ * nua_handle_unref().
  */
 nua_handle_t *nua_handle(nua_t *nua, nua_hmagic_t *hmagic,
 			 tag_type_t tag, tag_value_t value, ...)
@@ -321,6 +334,32 @@ void nua_handle_bind(nua_handle_t *nh, nua_hmagic_t *hmagic)
     nh->nh_magic = hmagic;
 }
 
+/** Fetch a callback context from an operation handle. 
+ *
+ * @param nh          Pointer to operation handle
+ *
+ * @return
+ *     Pointer to callback context
+ *
+ * @par Related tags:
+ *     none
+ *
+ * @par Events:
+ *     none
+ *
+ * @NEW_1_12_4.
+ */
+nua_hmagic_t *nua_handle_magic(nua_handle_t *nh)
+{
+  nua_hmagic_t *magic = NULL;
+  enter;
+
+  if (NH_IS_VALID(nh))
+    magic = nh->nh_magic;
+  
+  return magic;
+}
+
 /* ---------------------------------------------------------------------- */
 
 /** Check if operation handle is used for INVITE
@@ -346,8 +385,8 @@ int nua_handle_has_invite(nua_handle_t const *nh)
 
 /**Check if operation handle has active event subscriptions. 
  *
- * Active subscription can be established either by nua_subscribe 
- * or nua_refer() calls. 
+ * Active subscription can be established either by nua_subscribe() or
+ * nua_refer() calls.
  *
  * @param nh          Pointer to operation handle
  *
@@ -368,8 +407,12 @@ int nua_handle_has_events(nua_handle_t const *nh)
 
 /** Check if operation handle has active registrations
  *
- * Either REGISTER operation is ongoing or NUA stack is expected to 
- * refresh in the future.
+ * A registration is active when either when a REGISTER operation is going
+ * on or when it has successfully completed so that @nua stack is expected to
+ * refresh the registration in the future. Normally, a handle has active
+ * registration after nua_register() until nua_unregister() completes,
+ * unless the initial nua_register() had either expiration time of 0 or it
+ * had SIPTAG_CONTACT(NULL) as an argument.
  *
  * @param nh          Pointer to operation handle
  *
@@ -382,6 +425,8 @@ int nua_handle_has_events(nua_handle_t const *nh)
  *
  * @par Events:
  *     none
+ *
+ * @sa nua_register(), nua_unregister(), #nua_r_register, #nua_r_unregister
  */
 int nua_handle_has_registrations(nua_handle_t const *nh)
 {
@@ -419,18 +464,12 @@ int nua_handle_has_subscribe(nua_handle_t const *nh)
  *
  * @par Events:
  *     none
-
  */
 int nua_handle_has_register(nua_handle_t const *nh)
 {
   return nh ? nh->nh_has_register : 0;
 }
 
-int nua_handle_has_streaming(nua_handle_t const *nh)
-{
-  return nh ? nh->nh_has_streaming : 0;
-}
-
 /** Check if operation handle has an active call 
  *
  * @param nh          Pointer to operation handle
@@ -446,21 +485,22 @@ int nua_handle_has_streaming(nua_handle_t const *nh)
  */
 int nua_handle_has_active_call(nua_handle_t const *nh)
 {
-  return nh ? nh->nh_ss->ss_active : 0;
+  return nh ? nh->nh_active_call : 0;
 }
 
 /** Check if operation handle has a call on hold 
  *
  * Please note that this status is not affected by remote end putting 
  * this end on hold. Remote end can put each media separately on hold 
- * and status is reflected on #SOATAG_ACTIVE_AUDIO, #SOATAG_ACTIVE_VIDEO 
- * and #SOATAG_ACTIVE_CHAT tag values in nua_i_active event.
+ * and status is reflected on SOATAG_ACTIVE_AUDIO(), SOATAG_ACTIVE_VIDEO() 
+ * and SOATAG_ACTIVE_CHAT() tag values in #nua_i_state event.
  *
  * @param nh          Pointer to operation handle
  *
  * @retval 0  if no call on hold in operation or operation handle is invalid 
  * @retval 1  if operation has call on hold, for example nua_invite() or 
- *            nua_update() has been called with NUTAG_HOLD() with != 0 argument.
+ *            nua_update() has been called with SOATAG_HOLD() with non-NULL
+ *            argument.
  *
  * @par Related tags:
  *     none
@@ -470,7 +510,7 @@ int nua_handle_has_active_call(nua_handle_t const *nh)
  */
 int nua_handle_has_call_on_hold(nua_handle_t const *nh)
 {
-  return nh ? nh->nh_ss->ss_hold_remote : 0;
+  return nh ? nh->nh_hold_remote : 0;
 }
 
 /** Get the remote address (From/To header) of operation handle
@@ -515,6 +555,7 @@ sip_to_t const *nua_handle_local(nua_handle_t const *nh)
   return nh ? nh->nh_ds->ds_local : NULL;
 }
 
+/* Documented with nua_stack_set_params() */
 void nua_set_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
 {
   ta_list ta;
@@ -527,6 +568,7 @@ void nua_set_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
   ta_end(ta);
 }
 
+/* Documented with nua_stack_get_params() */
 void nua_get_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
 {
   ta_list ta;
@@ -551,11 +593,13 @@ void nua_get_params(nua_t *nua, tag_type_t tag, tag_value_t value, ...)
     SU_DEBUG_1(("nua: " #event " with invalid handle %p\n", nh));	\
   }
 
+/* Documented with nua_stack_set_params() */
 void nua_set_hparams(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_set_params, tag, value);
 }
 
+/* Documented with nua_stack_get_params() */
 void nua_get_hparams(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_get_params, tag, value);
@@ -572,178 +616,53 @@ void nua_unregister(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
   NUA_SIGNAL(nh, nua_r_unregister, tag, value);
 }
 
-/** Place a call using SIP INVITE method. 
- *
- * By default creates a media session, includes its description as 
- * SDP and send the request to the recipient. Upon receiving the 
- * response it will active the media session and establish the call. 
- * 
- * Incomplete call can be hung-up with nua_cancel(). Completed call can be
- * hung-up with nua_bye().
- *
- * Optionally 
- * - uses early media if NUTAG_EARLY_MEDIA() tag is used with non zero value
- * - media parameters can be set by NUTAG_MEDIA_* tags
- * - if NUTAG_MEDIA_ENABLE() tag is used with value zero then the soa is 
- *   not used and application must create the SDP
- * - nua_invite() can be used to change call status: 
- *   - #SOATAG_HOLD tag listing the media put on hold or with value "*" sets
- *     the call on hold
- *   - if new media path is given either new media parameters are taken in 
- *     use or new media is added to session.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    NUTAG_URL() \n
- *    NUTAG_HOLD() \n
- *    NUTAG_NOTIFY_REFER() \n
- *    NUTAG_REFER_PAUSE() \n
- *    NUTAG_INVITE_TIMER() \n
- *    NUTAG_MEDIA_FEATURES() \n
- *    NUTAG_MEDIA_ENABLE() \n
- *    SOATAG_HOLD() \n
- *    SOATAG_AF() \n
- *    SOATAG_ADDRESS() \n
- *    SOATAG_USER_SDP() or SOATAG_USER_SDP_STR() \n
- *    tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_invite \n
- *    #nua_i_state \n
- *    #nua_i_active \n
- *    #nua_i_media_error \n
- *    #nua_i_fork \n
- *
- * \sa nua_handle_has_active_call() \n
- *     nua_handle_has_call_on_hold()\n
- *     nua_handle_has_invite() \n
- *     nua_update() \n
- *     nua_info() \n 
- *     nua_cancel() \n
- *     nua_bye()
- */
+/* Documented with nua_stack_invite() */
 void nua_invite(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_invite, tag, value);
 }
 
-/** Acknowledge a succesfull response to INVITE request.
- *
- * Acknowledge a successful response to INVITE request 
- * with SIP ACK message. This function is need only if 
- * NUTAG_AUTOACK() parameter has been cleared.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_i_media_error \n
- *    #nua_i_active
- */
+/* Documented with nua_stack_ack() */
 void nua_ack(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_ack, tag, value);
 }
 
-/** Hangdown a call.
- *
- * Hangdown a call using SIP BYE method. Also the media session 
- * associated with the call is terminated. 
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    none
- *
- * @par Events:
- *    #nua_r_bye \n
- *    #nua_i_media_error
- */
+/* Documented with nua_stack_bye() */
 void nua_bye(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_bye, tag, value);
 }
 
-/** Cancel an INVITE operation 
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_cancel
- */
+/* Documented with nua_stack_cancel() */
 void nua_cancel(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_cancel, tag, value);
 }
 
-/** Query capabilities from server 
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_options
- *
- */
+/* Documented with nua_stack_options() */
 void nua_options(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_options, tag, value);
 }
 
-/** Send an instant message. 
- *
- * Send an instant message using SIP MESSAGE method.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_message
- */
+/* Documented with nua_stack_message() */
 void nua_message(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_message, tag, value);
 }
 
+/* Documented with nua_stack_method() */
+void nua_method(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
+{
+  NUA_SIGNAL(nh, nua_r_method, tag, value);
+}
+
 /** Send a chat message. 
  *
  * A chat channel can be established during call setup using "message" media. 
- * An active chat channel is indicated using nua_i_active event containing 
- * #SOATAG_ACTIVE_CHAT tag. Chat messages can be sent using this channel with 
+ * An active chat channel is indicated using #nua_i_state event containing 
+ * SOATAG_ACTIVE_CHAT() tag. Chat messages can be sent using this channel with 
  * nua_chat() function. Currently this is implemented using SIP MESSAGE 
  * requests but in future MSRP (message session protocol) will replace it.
 *
@@ -754,11 +673,11 @@ void nua_message(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
  *    nothing
  *
  * @par Related Tags:
- *    #SIPTAG_CONTENT_TYPE \n
- *    #SIPTAG_PAYLOAD      \n
- *    #SIPTAG_FROM         \n
- *    #SIPTAG_TO           \n
- *    use of other SIP tags' is deprecated
+ *    SIPTAG_CONTENT_TYPE() \n
+ *    SIPTAG_PAYLOAD()      \n
+ *    SIPTAG_FROM()         \n
+ *    SIPTAG_TO()           \n
+ *    Use of other SIP tags is deprecated
  *
  * @par Events:
  *    #nua_r_chat
@@ -768,80 +687,26 @@ void nua_chat(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
   NUA_SIGNAL(nh, nua_r_chat, tag, value);
 }
 
-/** Subscribe a SIP event. 
- *
- * Subscribe a SIP event using the SIP SUBSCRIBE request. If the 
- * SUBSCRBE is successful a subscription state is established and 
- * the subscription is refreshed regularly. The refresh requests will
- * generate #nua_r_subscribe events.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    NUTAG_URL()
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_subscribe \n
- *    #nua_i_notify
- */
+/* Documented with nua_stack_subscribe() */
 void nua_subscribe(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_subscribe, tag, value);
 }
 
-/** Unsubscribe an event. 
- *
- * Unsubscribe an active or pending subscription with SUBSCRIBE request 
- * containing Expires: header with value 0. The dialog associated with 
- * subscription will be destroyed if there is no other subscriptions or 
- * call using this dialog.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    #SIPTAG_EVENT \n
- *    Tags in <sip_tag.h> except #SIPTAG_EXPIRES or #SIPTAG_EXPIRES_STR
- *
- * @par Events:
- *    #nua_r_unsubscribe 
- */
+/* Documented with nua_stack_subscribe() */
 void nua_unsubscribe(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_unsubscribe, tag, value);
 }
 
-/** Send a SIP NOTIFY request message. 
- *
- * This function is used when the application implements itself 
- * the subscription state machine. The application must provide 
- * valid @b Subscription-State and @b Event headers using SIP tags.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_notify
- */
+/* Documented with nua_stack_notify() */
 void nua_notify(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_notify, tag, value);
 }
 
+/* nua_r_notify is documented with process_response_to_notify() */
+
 /** Create an event server. 
  *
  * This function create an event server taking care of sending NOTIFY 
@@ -858,10 +723,10 @@ void nua_notify(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
  *
  * @par Related Tags:
  *    NUTAG_URL() \n
- *    #SIPTAG_EVENT or #SIPTAG_EVENT_STR \n
- *    #SIPTAG_CONTENT_TYPE or SIPTAG_CONTENT_TYPE_STR \n
- *    #SIPTAG_PAYLOAD or #SIPTAG_PAYLOAD_STR \n
- *    #SIPTAG_ACCEPT or #SIPTAG_ACCEPT_STR \n
+ *    SIPTAG_EVENT() or SIPTAG_EVENT_STR() \n
+ *    SIPTAG_CONTENT_TYPE() or SIPTAG_CONTENT_TYPE_STR() \n
+ *    SIPTAG_PAYLOAD() or SIPTAG_PAYLOAD_STR() \n
+ *    SIPTAG_ACCEPT() or SIPTAG_ACCEPT_STR() \n
  *
  * @par Events:
  *    #nua_r_notify
@@ -883,44 +748,22 @@ void nua_notifier(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
  *    nothing
  *
  * @par Related Tags:
- *    #SIPTAG_EVENT \n
- *    #SIPTAG_CONTENT_TYPE \n
- *    #SIPTAG_PAYLOAD \n
- *    #NEATAG_REASON
+ *    SIPTAG_EVENT() \n
+ *    SIPTAG_CONTENT_TYPE() \n
+ *    SIPTAG_PAYLOAD() \n
+ *    NEATAG_REASON()
  *
  * @par Events:
  *    #nua_r_terminate
+ *
+ * @sa nua_notifier(), nua_authorize().
  */
 void nua_terminate(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_terminate, tag, value);
 }
 
-/** Transfer a call. 
- * 
- * Send a REFER request asking the recipient to transfer the call. The REFER
- * request also establishes a subscription to the "refer" event. The "refer"
- * event will have an "id" parameter, which has the value of CSeq number in
- * the REFER request. After initiating the REFER request, the nua engine
- * sends application a nua_r_refer event with status 100 and tag
- * SIPTAG_EVENT() containing a matching event header.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    NUTAG_URL() \n
- *    Tags in <sip_tag.h>
- *
- * @par Events:
- *    #nua_r_refer \n
- *    #nua_i_notify
- *
- * @sa @RFC3515
- */
+/* Documented with nua_stack_refer() */
 void nua_refer(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_refer, tag, value);
@@ -938,74 +781,19 @@ void nua_unpublish(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
   NUA_SIGNAL(nh, nua_r_unpublish, tag, value);
 }
 
-/** Send an INFO request. 
- *
- * INFO is used to send call related information like DTMF 
- * digit input events. See @RFC2976.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sip_tag.h>.
- *
- * @par Events:
- *    #nua_r_info
- */
+/* Documented with nua_stack_info() */
 void nua_info(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_info, tag, value);
 }
 
-
-/** Send a PRACK request. 
- *
- * PRACK is used to acknowledge receipt of 100rel responses. See @RFC3262.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    Tags in <sofia-sip/soa_tag.h>, <sofia-sip/sip_tag.h>.
- *
- * @par Events:
- *    #nua_r_prack
- */
+/* Documented with nua_stack_prack() */
 void nua_prack(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_prack, tag, value);
 }
 
-
-/** Update a session. 
- * 
- * Update a session using SIP UPDATE method. See @RFC3311.
- *
- * Update method can be used when the session has been established with
- * INVITE. It's mainly used during the session establishment when
- * preconditions are used (@RFC3312). It can be also used during the call if
- * no user input is needed for offer/answer negotiation.
- *
- * @param nh              Pointer to operation handle
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    same as nua_invite()
- *
- * @par Events:
- *    #nua_r_update \n
- *    #nua_i_media_error \n
- *    #nua_i_media_update
- */
+/* Documented with nua_stack_update() */
 void nua_update(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
   NUA_SIGNAL(nh, nua_r_update, tag, value);
@@ -1015,7 +803,7 @@ void nua_update(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
  *
  * - 401 / 407 response with www-authenticate header/ proxy-authenticate header
  * - application should provide stack with username&password for each realm
- * with NUTAG_AUTH() tag
+ *   with NUTAG_AUTH() tag
  * - restarts operation
  *
  * @param nh              Pointer to operation handle
@@ -1037,12 +825,12 @@ void nua_authenticate(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 
 /** Authorize a subscriber.
  *
- * After creating a local presence server by nua_notifier(), an
- * incoming subscriber launches nua_i_subscription event. Subscriber
- * can be authorized in this application callback.
- *
- * NUTAG_SUB() tag
- * NUTAG_SUBSTATE() tag
+ * After creating a local presence server by nua_notifier(), an incoming
+ * SUBSCRIBE request causes #nua_i_subscription event. Each subscriber is
+ * identified with NEATAG_SUB() tag in the #nua_i_subscription event. 
+ * Application can either authorize the subscriber with
+ * NUTAG_SUBSTATE(#nua_substate_active) or terminate the subscription with
+ * NUTAG_SUBSTATE(#nua_substate_terminated).
  *
  * @param nh              Pointer to operation handle
  * @param tag, value, ... List of tagged parameters
@@ -1051,11 +839,13 @@ void nua_authenticate(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
  *    nothing
  *
  * @par Related Tags:
- *    NUTAG_SUB()
+ *    NEATAG_SUB() \n
  *    NUTAG_SUBSTATE()
  *
  * @par Events:
- *    (any operation events)
+ *    #nua_i_subscription
+ *
+ * @sa nua_notifier(), nua_terminate()
  */
 void nua_authorize(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
 {
@@ -1068,31 +858,8 @@ void nua_redirect(nua_handle_t *nh, tag_type_t tag, tag_value_t value, ...)
   NUA_SIGNAL(nh, nua_r_redirect, tag, value);
 }
 
-/** Respond with given status. 
- * 
- * The INVITE requests should be responded with 
- * #nua_intive_respond() function because there 
- * might be another request going on besides INVITE.
- *
- * @param nh              Pointer to operation handle
- * @param status          SIP response status (see RFCs of SIP)
- * @param phrase          free text (default response phrase used if NULL)
- * @param tag, value, ... List of tagged parameters
- *
- * @return 
- *    nothing
- *
- * @par Related Tags:
- *    #SOATAG_ADDRESS \n
- *    #SOATAG_AF \n
- *    #SOATAG_HOLD \n
- *    Tags in <sip_tag.h>.
- *
- * @par Events:
- *    #nua_i_active \n
- *    #nua_i_media_error \n
- *    #nua_i_error
- */
+/* Documented with nua_stack_respond() */
+
 void nua_respond(nua_handle_t *nh,
 		 int status, char const *phrase,
 		 tag_type_t tag, tag_value_t value,
@@ -1114,9 +881,15 @@ void nua_respond(nua_handle_t *nh,
 
 /** Destroy a handle 
  *
- * Destroy an operation handle and asks stack to discard resources 
- * and ongoing sessions and transactions associated with this handle. 
- * For example calls are terminated with BYE request.
+ * Terminate the protocol state associated with an operation handle. The
+ * stack discards resources and terminates the ongoing dialog usage,
+ * sessions and transactions associated with this handle. For example, calls
+ * are terminated with BYE request. Also, the reference count for the handle
+ * is also decremented.
+ *
+ * The handles use reference counting for memory management. In order to
+ * make it more convenient for programmer, nua_handle_destroy() decreases
+ * the reference count, too.
  *
  * @param nh              Pointer to operation handle
  *
@@ -1128,6 +901,9 @@ void nua_respond(nua_handle_t *nh,
  *
  * @par Events:
  *    none
+ *
+ * @sa nua_handle(), nua_handle_bind(), nua_handle_ref(), nua_handle_unref(),
+ * nua_unregister(), nua_unpublish(), nua_unsubscribe(), nua_bye().
  */
 void nua_handle_destroy(nua_handle_t *nh)
 {
@@ -1139,7 +915,6 @@ void nua_handle_destroy(nua_handle_t *nh)
   }
 }
 
-
 /*# Send a request to the protocol thread */
 void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
 		nua_event_t event,
@@ -1147,7 +922,7 @@ void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
 		tag_type_t tag, tag_value_t value, ...)
 {
   su_msg_r sumsg = SU_MSG_R_INIT;
-  int len, xtra, e_len, l_len = 0, l_xtra = 0;
+  size_t len, xtra, e_len, l_len = 0, l_xtra = 0;
   ta_list ta;
 
   if (nua == NULL || (nua->nua_shutdown_started && event != nua_r_shutdown))
@@ -1159,7 +934,7 @@ void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
   len = tl_len(ta_args(ta));
   xtra = tl_xtra(ta_args(ta), len);
 
-  if (su_msg_create(sumsg, nua->nua_server, nua->nua_client,
+  if (su_msg_create(sumsg, nua->nua_server, su_task_null,
 		    nua_stack_signal,
 		    e_len + len + l_len + xtra + l_xtra) == 0) {
     event_t *e = su_msg_data(sumsg);
@@ -1171,7 +946,7 @@ void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
 
     t = tl_dup(t, ta_args(ta), &b);
 
-    assert(tend == t); assert(b == bend);
+    assert(tend == t); (void)tend; assert(b == bend); (void)bend;
 
     e->e_always = always;
     e->e_event = event;
@@ -1183,7 +958,8 @@ void nua_signal(nua_t *nua, nua_handle_t *nh, msg_t *msg, int always,
       nua_handle_unref(nh);
   } 
   else {
-    assert(0);
+    /* XXX  - we should return error code to application */
+    assert(ENOMEM == 0);
   }
 
   ta_end(ta);
@@ -1209,7 +985,7 @@ void nua_event(nua_t *root_magic, su_msg_r sumsg, event_t *e)
       SU_DEBUG_9(("nua(%p): freed by application\n", nh));
     }
     if (e->e_msg)
-      msg_destroy(e->e_msg);
+      msg_destroy(e->e_msg), e->e_msg = NULL;
     return;
   }
 
@@ -1242,11 +1018,23 @@ void nua_event(nua_t *root_magic, su_msg_r sumsg, event_t *e)
     return;
 
   if (e->e_msg)
-    msg_destroy(e->e_msg);
+    msg_destroy(e->e_msg), e->e_msg = NULL;
 
   su_msg_destroy(nua->nua_current);
 }
 
+/** Get current request message. @NEW_1_12_4. */
+msg_t *nua_current_request(nua_t const *nua)
+{
+  return nua && nua->nua_current ? su_msg_data(nua->nua_current)->e_msg : NULL;
+}
+
+/** Get request message from saved nua event. @NEW_1_12_4. */
+msg_t *nua_saved_event_request(nua_saved_event_t const *saved)
+{
+  return saved ? su_msg_data(saved)->e_msg : NULL;
+}
+
 /** Save nua event and its arguments */
 int nua_save_event(nua_t *nua, nua_saved_event_t return_saved[1])
 {
@@ -1275,7 +1063,7 @@ void nua_destroy_event(nua_saved_event_t saved[1])
     nua_handle_t *nh = e->e_nh;
 
     if (e->e_msg)
-      msg_destroy(e->e_msg);
+      msg_destroy(e->e_msg), e->e_msg = NULL;
 
     if (nh && !NH_IS_DEFAULT(nh) && nua_handle_unref(nh)) {
       SU_DEBUG_9(("nua(%p): freed by application\n", nh));
@@ -1284,3 +1072,88 @@ void nua_destroy_event(nua_saved_event_t saved[1])
     su_msg_destroy(saved);
   }
 }
+
+/* ---------------------------------------------------------------------- */
+
+struct nua_stack_handle_make_replaces_args {
+  sip_replaces_t *retval;
+  nua_handle_t *nh;
+  su_home_t *home;
+  int early_only;
+};
+
+static int nua_stack_handle_make_replaces_call(void *arg)
+{
+  struct nua_stack_handle_make_replaces_args *a = arg;
+
+  a->retval = nua_stack_handle_make_replaces(a->nh, a->home, a->early_only);
+
+  return 0;
+}
+
+
+/**Generate a @Replaces header for handle.
+ *
+ * @since New in @VERSION_1_12_4.
+ *
+ * @sa nua_handle_by_replaces(), @Replaces, @RFC3891, nua_refer(),
+ * #nua_i_refer, @ReferTo, nta_leg_make_replaces()
+ */
+sip_replaces_t *nua_handle_make_replaces(nua_handle_t *nh, 
+					 su_home_t *home,
+					 int early_only)
+{
+  if (nh && nh->nh_valid && nh->nh_nua) {
+    struct nua_stack_handle_make_replaces_args a = { NULL, nh, home, early_only };
+
+    if (su_task_execute(nh->nh_nua->nua_server, 
+			nua_stack_handle_make_replaces_call, (void *)&a, 
+			NULL) == 0) {
+      return a.retval;
+    }
+  }
+  return NULL;
+}
+
+struct nua_stack_handle_by_replaces_args {
+  nua_handle_t *retval;
+  nua_t *nua;
+  sip_replaces_t const *r;
+};
+
+static int nua_stack_handle_by_replaces_call(void *arg)
+{
+  struct nua_stack_handle_by_replaces_args *a = arg;
+
+  a->retval = nua_stack_handle_by_replaces(a->nua, a->r);
+
+  return 0;
+}
+
+/** Obtain a new reference to an existing handle based on @Replaces header.
+ *
+ * @since New in @VERSION_1_12_4.
+ *
+ * @note 
+ * You should release the reference with nua_handle_unref() when you are
+ * done with handle.
+ *
+ * @sa nua_handle_make_replaces(), @Replaces, @RFC3891, nua_refer(),
+ * #nua_i_refer, @ReferTo, nta_leg_by_replaces()
+ */
+nua_handle_t *nua_handle_by_replaces(nua_t *nua, sip_replaces_t const *r)
+{
+  if (nua) {
+    struct nua_stack_handle_by_replaces_args a = { NULL, nua, r };
+
+    if (su_task_execute(nua->nua_server, 
+			nua_stack_handle_by_replaces_call, (void *)&a, 
+			NULL) == 0) {
+      nua_handle_t *nh = a.retval;
+
+      if (nh && !NH_IS_DEFAULT(nh) && nh->nh_valid)
+	return nua_handle_ref(nh);
+    }
+  }
+  return NULL;
+}
