@@ -22,7 +22,8 @@
  *
  */
 
-#ifndef NUA_DIALOG_H /** Defined when <nua_dialog.h> has been included. */
+#ifndef NUA_DIALOG_H
+/** Defined when <nua_dialog.h> has been included. */
 #define NUA_DIALOG_H
 
 /**@IFILE nua_dialog.h 
@@ -47,8 +48,88 @@ typedef NUA_OWNER_T nua_owner_t;
 #include <sofia-sip/nta.h>
 #endif
 
+typedef struct nua_server_request nua_server_request_t; 
+typedef struct nua_client_request nua_client_request_t; 
+
+/** Respond to an incoming request. */
+typedef int nua_server_respond_f(nua_server_request_t *, tagi_t const *);
+
+/** Restart an outgoing request. */
+typedef void nua_creq_restart_f(nua_owner_t *, tagi_t *tags);
+
+/** Server side transaction */
+struct nua_server_request {
+  struct nua_server_request *sr_next, **sr_prev;
+
+  nua_owner_t *sr_owner;	/**< Backpointer to handle */
+  nua_dialog_usage_t *sr_usage;	/**< Backpointer to usage */
+
+  /** When the application responds to an request with
+   * nua_respond(), the sr_respond() is called
+   */
+  nua_server_respond_f *sr_respond;
+  
+  nta_incoming_t *sr_irq;	/**< Server transaction object */
+  msg_t *sr_msg;		/**< Request message */
+
+  sip_method_t sr_method;	/**< Request method */
+  int sr_status;		/**< Status code */
+  char const *sr_phrase;	/**< Status phrase */
+
+  unsigned sr_auto:1;		/**< Autoresponse - no event has been sent */
+  unsigned sr_initial:1;	/**< Handle was created by this request */
+
+  /* Flags used with offer-answer */
+  unsigned sr_offer_recv:1;	/**< We have received an offer */
+  unsigned sr_answer_sent:2;	/**< We have answered (reliably, if >1) */
+
+  unsigned sr_offer_sent:1;	/**< We have offered SDP */
+  unsigned sr_answer_recv:1;	/**< We have received SDP answer */
+};
+
+#define SR_INIT(sr) \
+  (memset((sr), 0, sizeof (sr)[0]), SR_STATUS1((sr), SIP_100_TRYING), sr)
+
+#define SR_STATUS(sr, status, phrase) \
+  ((sr)->sr_phrase = (phrase), (sr)->sr_status = (status))
+
+#define SR_STATUS1(sr, statusphrase)					\
+  sr_status(sr, statusphrase)
+
+su_inline 
+int sr_status(nua_server_request_t *sr, int status, char const *phrase)
+{
+  return (void)(sr->sr_phrase = phrase), (sr->sr_status = status);
+}
+
+struct nua_client_request
+{
+  nua_client_request_t *cr_next;        /**< Linked list of requests */
+  /*nua_event_t*/ int cr_event;		/**< Request event */
+  nua_creq_restart_f *cr_restart;
+  nta_outgoing_t     *cr_orq;
+  msg_t              *cr_msg;
+  nua_dialog_usage_t *cr_usage;
+  unsigned short      cr_retry_count;   /**< Retry count for this request */
+
+  /* Flags used with offer-answer */
+  unsigned short      cr_answer_recv;   /**< Recv answer in response 
+					 *  with this status.
+					 */
+  unsigned            cr_offer_sent:1;  /**< Sent offer in this request */
+
+  unsigned            cr_offer_recv:1;  /**< Recv offer in a response */
+  unsigned            cr_answer_sent:1; /**< Sent answer in (PR)ACK */
+
+  unsigned            cr_has_contact:1; /**< Request has application contact */
+};
+
+
 struct nua_dialog_state
 {
+  nua_client_request_t ds_cr[1];
+  nua_server_request_t *ds_sr;
+
   /** Dialog usages. */
   nua_dialog_usage_t     *ds_usage;
 
@@ -59,6 +140,9 @@ struct nua_dialog_state
   unsigned ds_has_session:1;	/**< We have session */
   unsigned ds_has_register:1;	/**< We have registration */
   unsigned ds_has_publish:1;	/**< We have publish */
+
+  unsigned ds_has_referrals:1;	/**< We have (or have had) referrals */
+
   unsigned :0;
 
   unsigned ds_has_events;	/**< We have events */
@@ -86,6 +170,7 @@ typedef void nh_pending_f(nua_owner_t *,
 			  nua_dialog_usage_t *du,
 			  sip_time_t now);
 
+/** Virtual function pointer table for dialog usage. */
 typedef struct {
   unsigned usage_size, usage_class_size;
   int (*usage_add)(nua_owner_t *, 
@@ -98,10 +183,14 @@ typedef struct {
   void (*usage_peer_info)(nua_dialog_usage_t *du,
 			  nua_dialog_state_t const *ds,
 			  sip_t const *sip);
-  void (*usage_refresh)(nua_owner_t *, nua_dialog_usage_t *, sip_time_t now);
-  int (*usage_shutdown)(nua_owner_t *, nua_dialog_usage_t *);
+  void (*usage_refresh)(nua_owner_t *, nua_dialog_state_t *ds,
+			nua_dialog_usage_t *, sip_time_t now);
+  int (*usage_shutdown)(nua_owner_t *, nua_dialog_state_t *ds, 
+			nua_dialog_usage_t *);
 } nua_usage_class;
 
+
+/** Base structure for dialog usage. */
 struct nua_dialog_usage {
   nua_dialog_usage_t *du_next;
   nua_usage_class const *du_class;
@@ -111,17 +200,16 @@ struct nua_dialog_usage {
   unsigned     du_shutdown:1;	        /**< Shutdown in progress */
   unsigned:0;
 
-  /** Pending operation.
-   *
-   * The du_pending() is called 
-   * a) when current time sip_now() will soon exceed du_refresh (now > 1)
-   * b) when operation is restarted (now is 1)
-   * c) when usage is destroyed (now is 0)
+  /** When usage expires.
+   * Non-zero if the usage is established, SIP_TIME_MAX if there no
+   * expiration time.
    */
-  nh_pending_f   *du_pending;
-  sip_time_t      du_refresh;		/**< When execute du_pending */
+  sip_time_t      du_expires;		
+
+  sip_time_t      du_refresh;		/**< When to refresh */
 
   sip_event_t const *du_event;		/**< Event of usage */
+
   msg_t *du_msg;			/**< Template message */
 };
 
@@ -131,6 +219,9 @@ void nua_dialog_uas_route(nua_owner_t *, nua_dialog_state_t *ds,
 			  sip_t const *sip, int rtag);
 void nua_dialog_store_peer_info(nua_owner_t *, nua_dialog_state_t *ds,
 				sip_t const *sip);
+int nua_dialog_remove(nua_owner_t *own,
+		      nua_dialog_state_t *ds,
+		      nua_dialog_usage_t *usage);
 
 char const *nua_dialog_usage_name(nua_dialog_usage_t const *du);
 
@@ -147,18 +238,28 @@ void nua_dialog_usage_remove(nua_owner_t *,
 			     nua_dialog_state_t *ds,
 			     nua_dialog_usage_t *du);
 
+void nua_dialog_deinit(nua_owner_t *own,
+		       nua_dialog_state_t *ds);
+
 void nua_dialog_terminated(nua_owner_t *,
 			   struct nua_dialog_state *ds,
 			   int status,
 			   char const *phrase);
 
+void nua_dialog_usage_set_expires(nua_dialog_usage_t *du, unsigned delta);
+
 void nua_dialog_usage_set_refresh(nua_dialog_usage_t *du, unsigned delta);
 
+void nua_dialog_usage_refresh_range(nua_dialog_usage_t *du, 
+				    unsigned min, unsigned max);
+
+void nua_dialog_usage_reset_refresh(nua_dialog_usage_t *du);
+
 void nua_dialog_usage_refresh(nua_owner_t *owner,
+			      nua_dialog_state_t *ds,
 			      nua_dialog_usage_t *du, 
 			      sip_time_t now);
 
-
 static inline
 int nua_dialog_is_established(nua_dialog_state_t const *ds)
 {
@@ -182,4 +283,20 @@ nua_dialog_usage_t *nua_dialog_usage_public(void const *p)
 #define nua_dialog_usage_public(p) ((p) ? (nua_dialog_usage_t*)(p) - 1 : NULL)
 #endif
 
+/* ---------------------------------------------------------------------- */
+
+void nua_server_request_destroy(nua_server_request_t *sr);
+
+int nua_server_respond(nua_server_request_t *sr,
+		       int status, char const *phrase,
+		       tag_type_t tag, tag_value_t value, ...);
+
+msg_t *nua_server_response(nua_server_request_t *sr,
+			   int status, char const *phrase,
+			   tag_type_t tag, tag_value_t value, ...);
+
+int nua_default_respond(nua_server_request_t *sr,
+			tagi_t const *tags);
+
+
 #endif /* NUA_DIALOG_H */
