@@ -23,7 +23,12 @@
  */
 
 /**@CFILE nua_subnotref.c
- * @brief SUBSCRIBE, NOTIFY and REFER methods
+ * @brief Subscriber (event watcher)
+ *
+ * This file contains implementation SUBSCRIBE UAC, NOTIFY UAS, REFER UAC.
+ * The implementation of SUBSCRIBE UAS, NOTIFY UAC and REFER UAS is in
+ * nua_notifier.c.
+ * Alternative implementation using nea is in nua_event_server.c.
  *
  * @author Pekka Pessi <Pekka.Pessi@nokia.com>
  *
@@ -51,12 +56,15 @@
 #include "nua_stack.h"
 
 /* ---------------------------------------------------------------------- */
-/* Subcribe event usage */
+/* Subcriber event usage */
 
 struct event_usage
 {
   enum nua_substate  eu_substate;	/**< Subscription state */
   sip_time_t eu_expires;	        /**< Proposed expiration time */
+  unsigned eu_notified;		        /**< Number of NOTIFYs received */
+  unsigned eu_final_wait:1;	        /**< Waiting for final NOTIFY */
+  unsigned eu_no_id:1;		        /**< Do not use "id" (even if we have one) */
 };
 
 static char const *nua_subscribe_usage_name(nua_dialog_usage_t const *du);
@@ -66,6 +74,13 @@ static int nua_subscribe_usage_add(nua_handle_t *nh,
 static void nua_subscribe_usage_remove(nua_handle_t *nh, 
 				       nua_dialog_state_t *ds,
 				       nua_dialog_usage_t *du);
+static void nua_subscribe_usage_refresh(nua_handle_t *,
+					nua_dialog_state_t *,
+					nua_dialog_usage_t *,
+					sip_time_t);
+static int nua_subscribe_usage_shutdown(nua_handle_t *,
+					nua_dialog_state_t *,
+					nua_dialog_usage_t *);
 
 static nua_usage_class const nua_subscribe_usage[1] = {
   {
@@ -73,6 +88,9 @@ static nua_usage_class const nua_subscribe_usage[1] = {
     nua_subscribe_usage_add,
     nua_subscribe_usage_remove,
     nua_subscribe_usage_name,
+    NULL,
+    nua_subscribe_usage_refresh,
+    nua_subscribe_usage_shutdown
   }};
 
 static char const *nua_subscribe_usage_name(nua_dialog_usage_t const *du)
@@ -99,83 +117,81 @@ void nua_subscribe_usage_remove(nua_handle_t *nh,
   ds->ds_has_subscribes--;	
 }
 
-/* ---------------------------------------------------------------------- */
-/* Notify event usage */
-
-static char const *nua_notify_usage_name(nua_dialog_usage_t const *du);
-static int nua_notify_usage_add(nua_handle_t *nh, 
-				   nua_dialog_state_t *ds,
-				   nua_dialog_usage_t *du);
-static void nua_notify_usage_remove(nua_handle_t *nh, 
-				       nua_dialog_state_t *ds,
-				       nua_dialog_usage_t *du);
-
-static nua_usage_class const nua_notify_usage[1] = {
-  {
-    sizeof (struct event_usage), (sizeof nua_notify_usage),
-    nua_notify_usage_add,
-    nua_notify_usage_remove,
-    nua_notify_usage_name,
-  }};
-
-static char const *nua_notify_usage_name(nua_dialog_usage_t const *du)
-{
-  return "notify";
-}
-
-static 
-int nua_notify_usage_add(nua_handle_t *nh, 
-			   nua_dialog_state_t *ds,
-			   nua_dialog_usage_t *du)
-{
-  ds->ds_has_events++;
-  ds->ds_has_notifys++;
-  return 0;
-}
-
-static 
-void nua_notify_usage_remove(nua_handle_t *nh, 
-			       nua_dialog_state_t *ds,
-			       nua_dialog_usage_t *du)
-{
-  ds->ds_has_events--;	
-  ds->ds_has_notifys--;	
-}
-
 /* ====================================================================== */
 /* SUBSCRIBE */
 
-static void 
-  refresh_subscribe(nua_handle_t *nh, nua_dialog_usage_t *, sip_time_t now),
-  terminate_subscription(nua_handle_t *nh, nua_dialog_usage_t *, sip_time_t now);
+/** Subscribe to a SIP event. 
+ *
+ * Subscribe a SIP event using the SIP SUBSCRIBE request. If the 
+ * SUBSCRBE is successful a subscription state is established and 
+ * the subscription is refreshed regularly. The refresh requests will
+ * generate #nua_r_subscribe events.
+ *
+ * @param nh              Pointer to operation handle
+ * @param tag, value, ... List of tagged parameters
+ *
+ * @return 
+ *    nothing
+ *
+ * @par Related Tags:
+ *    NUTAG_URL()
+ *    Tags in <sip_tag.h>
+ *
+ * @par Events:
+ *    #nua_r_subscribe \n
+ *    #nua_i_notify
+ *
+ * @sa NUTAG_SUBSTATE(), @RFC3265
+ */
+
+/** Unsubscribe an event. 
+ *
+ * Unsubscribe an active or pending subscription with SUBSCRIBE request 
+ * containing Expires: header with value 0. The dialog associated with 
+ * subscription will be destroyed if there is no other subscriptions or 
+ * call using this dialog.
+ *
+ * @param nh              Pointer to operation handle
+ * @param tag, value, ... List of tagged parameters
+ *
+ * @return 
+ *    nothing
+ *
+ * @par Related Tags:
+ *    SIPTAG_EVENT() or SIPTAG_EVENT_STR() \n
+ *    Tags in <sip_tag.h> except SIPTAG_EXPIRES() or SIPTAG_EXPIRES_STR()
+ *
+ * @par Events:
+ *    #nua_r_unsubscribe 
+ *
+ * @sa NUTAG_SUBSTATE(), @RFC3265
+ */
+
 static int process_response_to_subscribe(nua_handle_t *nh,
 					 nta_outgoing_t *orq,
 					 sip_t const *sip);
 
+
 int
 nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
 		    tagi_t const *tags)
 {
-  nua_client_request_t *cr = nh->nh_cr;
+  nua_client_request_t *cr = nh->nh_ds->ds_cr;
   nua_dialog_usage_t *du = NULL;
   struct event_usage *eu;
   msg_t *msg;
   sip_t *sip;
 
-  if (nh->nh_special && nh->nh_special != nua_r_subscribe)
+  if (nua_stack_set_handle_special(nh, nh_has_subscribe, nua_r_subscribe) < 0)
     return UA_EVENT3(e, 500, "Invalid handle for SUBSCRIBE", 
 		     NUTAG_SUBSTATE(nua_substate_terminated));
   else if (cr->cr_orq)
-    return UA_EVENT2(e, 500, "Request already in progress");
+    return UA_EVENT2(e, 900, "Request already in progress");
 
   /* Initialize allow and auth */
-  nua_stack_init_handle(nua, nh, nh_has_subscribe, "NOTIFY", TAG_NEXT(tags));
+  nua_stack_init_handle(nua, nh, TAG_NEXT(tags));
 
-  if (nh->nh_has_subscribe)
-    /* We can re-use existing INVITE handle */
-    nh->nh_special = nua_r_subscribe;
-
-  msg = nua_creq_msg(nua, nh, cr, cr->cr_retry_count,
+  msg = nua_creq_msg(nua, nh, cr, 0,
 		     SIP_METHOD_SUBSCRIBE,
 		     NUTAG_USE_DIALOG(1),
 		     NUTAG_ADD_CONTACT(1),
@@ -185,22 +201,39 @@ nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
   if (sip) {
     sip_event_t *o = sip->sip_event;
 
-    if (e != nua_r_subscribe) {	/* Unsubscribe */
-      sip_add_make(msg, sip, sip_expires_class, "0");
-      du = nua_dialog_usage_get(nh->nh_ds, nua_subscribe_usage, o);
+    du = nua_dialog_usage_get(nh->nh_ds, nua_subscribe_usage, o);
 
-      if (du == NULL && o == NULL) {
-	du = nua_dialog_usage_get(nh->nh_ds, nua_subscribe_usage, NONE);
-	if (du && du->du_event)
-	  sip_add_dup(msg, sip, (sip_header_t *)du->du_event);
+    if (du == NULL && o == NULL)
+      du = nua_dialog_usage_get(nh->nh_ds, nua_subscribe_usage, NONE);
+
+    eu = nua_dialog_usage_private(du);
+
+    if (du && du->du_event && (o == NULL || (o->o_id && eu->eu_no_id))) {
+      if (eu->eu_no_id)		/* No id (XXX - nor other parameters) */
+	sip_add_make(msg, sip, sip_event_class, du->du_event->o_type);
+      else
+	sip_add_dup(msg, sip, (sip_header_t *)du->du_event);
+    }
+
+    if (e == nua_r_subscribe) {	
+      if (du == NULL)		/* Create dialog usage */
+	/* We allow here SUBSCRIBE without event */
+	du = nua_dialog_usage_add(nh, nh->nh_ds, nua_subscribe_usage, o);
+    }
+    else if (du) { /* Unsubscribe */
+      /* Embryonic subscription is just a placeholder */
+      if (eu->eu_substate == nua_substate_terminated ||
+	  eu->eu_substate == nua_substate_embryonic) {
+	nua_dialog_usage_remove(nh, nh->nh_ds, du);
+	msg_destroy(msg);
+	return UA_EVENT3(e, SIP_200_OK, 
+			 NUTAG_SUBSTATE(nua_substate_terminated),
+			 TAG_END());
       }
     }
-    else
-      /* We allow here SUBSCRIBE without event */
-      du = nua_dialog_usage_add(nh, nh->nh_ds, nua_subscribe_usage, o);
   }
 
-  /* Store supported features (eventlist) */
+  /* Store message template with supported features (eventlist) */
   if (du && sip) {
     if (du->du_msg)
       msg_destroy(du->du_msg);
@@ -211,6 +244,8 @@ nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
     cr->cr_orq = nta_outgoing_mcreate(nua->nua_nta,
 				      process_response_to_subscribe, nh, NULL,
 				      msg,
+				      TAG_IF(e != nua_r_subscribe,
+					     SIPTAG_EXPIRES_STR("0")),
 				      SIPTAG_END(), TAG_NEXT(tags));
 
   eu = nua_dialog_usage_private(du);
@@ -221,7 +256,7 @@ nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
     if (du == NULL)
       ;
     else if (du->du_ready)
-      substate = eu->eu_substate; /* We already  */
+      substate = eu->eu_substate; /* No change in subscription state  */
     else
       nua_dialog_usage_remove(nh, nh->nh_ds, du);
 
@@ -231,7 +266,7 @@ nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
 		     NUTAG_SUBSTATE(substate), TAG_END());
   }
 
-  du->du_pending = NULL;
+  nua_dialog_usage_reset_refresh(du); /* during SUBSCRIBE transaction */
   du->du_terminating = e != nua_r_subscribe; /* Unsubscribe or destroy */
 
   if (du->du_terminating)
@@ -244,6 +279,8 @@ nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
        [Event] packages MUST also define a
        default "Expires" value to be used if none is specified. */
     eu->eu_expires = 3600;
+
+  eu->eu_final_wait = 0;
     
   if (sip->sip_expires && sip->sip_expires->ex_delta == 0)
     du->du_terminating = 1;
@@ -257,14 +294,59 @@ nua_stack_subscribe(nua_t *nua, nua_handle_t *nh, nua_event_t e,
 
 static void restart_subscribe(nua_handle_t *nh, tagi_t *tags)
 {
-  nua_creq_restart(nh, nh->nh_cr, process_response_to_subscribe, tags);
+  nua_creq_restart(nh, nh->nh_ds->ds_cr, process_response_to_subscribe, tags);
 }
 
+/** @NUA_EVENT nua_r_subscribe
+ *
+ * Response to an outgoing SUBSCRIBE request.
+ *
+ * The SUBSCRIBE request may have been sent explicitly by nua_subscribe() or
+ * implicitly by NUA state machine.
+ *
+ * @param status response status code
+ *               (if the request is retried, @a status is 100, the @a
+ *               sip->sip_status->st_status contain the real status code
+ *               from the response message, e.g., 302, 401, or 407)
+ * @param phrase a short textual description of @a status code
+ * @param nh     operation handle associated with the subscription
+ * @param hmagic application context associated with the handle
+ * @param sip    response to SUBSCRIBE request or NULL upon an error
+ *               (status code is in @a status and 
+ *                descriptive message in @a phrase parameters)
+ * @param tags   NUTAG_SUBSTATE()
+ *
+ * @sa nua_subscribe(), @RFC3265
+ *
+ * @END_NUA_EVENT
+ */
+
+/** @NUA_EVENT nua_r_unsubscribe
+ *
+ * Response to an outgoing un-SUBSCRIBE.
+ *
+ * @param status response status code
+ *               (if the request is retried, @a status is 100, the @a
+ *               sip->sip_status->st_status contain the real status code
+ *               from the response message, e.g., 302, 401, or 407)
+ * @param phrase a short textual description of @a status code
+ * @param nh     operation handle associated with the subscription
+ * @param hmagic application context associated with the handle
+ * @param sip    response to SUBSCRIBE request or NULL upon an error
+ *               (status code is in @a status and 
+ *                descriptive message in @a phrase parameters)
+ * @param tags   NUTAG_SUBSTATE()
+ *
+ * @sa nua_unsubscribe(), @RFC3265
+ *
+ * @END_NUA_EVENT
+ */
+
 static int process_response_to_subscribe(nua_handle_t *nh,
 					 nta_outgoing_t *orq,
 					 sip_t const *sip)
 {
-  nua_client_request_t *cr = nh->nh_cr;
+  nua_client_request_t *cr = nh->nh_ds->ds_cr;
   nua_dialog_usage_t *du = cr->cr_usage; 
   struct event_usage *eu = nua_dialog_usage_private(du);
   int status = sip ? sip->sip_status->st_status : 408;
@@ -286,7 +368,7 @@ static int process_response_to_subscribe(nua_handle_t *nh,
     du->du_ready = 1;
     substate = eu->eu_substate;
     
-    if (cr->cr_event == nua_r_unsubscribe)
+    if (du->du_terminating)
       delta = 0;
     else
       /* If there is no expires header,
@@ -294,22 +376,34 @@ static int process_response_to_subscribe(nua_handle_t *nh,
       delta = sip_contact_expires(NULL, sip->sip_expires, sip->sip_date, 
 				  eu->eu_expires, now);
 
-    if (!win_messenger_enable)
-      nua_dialog_uac_route(nh, nh->nh_ds, sip, 1);
+    if (win_messenger_enable && !nua_dialog_is_established(nh->nh_ds)) {
+      /* Notify from messanger does not match with dialog tag */ 
+      nh->nh_ds->ds_remote_tag = su_strdup(nh->nh_home, "");
+    }
+
+    nua_dialog_uac_route(nh, nh->nh_ds, sip, 1);
     nua_dialog_store_peer_info(nh, nh->nh_ds, sip);
 
     if (delta > 0) {
       nua_dialog_usage_set_refresh(du, delta);
-      du->du_pending = refresh_subscribe;
+    }
+    else if (!eu->eu_notified) {
+      /* This is a fetch: subscription was really terminated
+	 but we wait 32 seconds for NOTIFY. */
+      delta = 64 * NTA_SIP_T1 / 1000;
+
+      if (win_messenger_enable)
+	delta = 4 * 60; 	/* Wait 4 minutes for NOTIFY from Messenger */
+
+      eu->eu_final_wait = 1;
+
+      /* Do not remove usage in nua_stack_process_response  */
+      cr->cr_usage = NULL;	
+
+      nua_dialog_usage_refresh_range(du, delta, delta);
     }
     else {
-      if (win_messenger_enable)
-	/* Wait 4 minutes for NOTIFY from Messenger */
-	du->du_refresh = now + 4 * 60; 
-      else
-	/* Wait 32 seconds for NOTIFY */
-	du->du_refresh = now + 64 * NTA_SIP_T1 / 1000;
-      du->du_pending = terminate_subscription;
+      eu->eu_substate = substate = nua_substate_terminated;
     }
   }
   else /* if (status >= 300) */ {
@@ -350,342 +444,146 @@ static int process_response_to_subscribe(nua_handle_t *nh,
 }
 
 /** Refresh subscription */
-void
-refresh_subscribe(nua_handle_t *nh, nua_dialog_usage_t *du, sip_time_t now)
+static void nua_subscribe_usage_refresh(nua_handle_t *nh,
+					nua_dialog_state_t *ds,
+					nua_dialog_usage_t *du,
+					sip_time_t now)
 {
   nua_t *nua = nh->nh_nua;
-  nua_client_request_t *cr = nh->nh_cr;
-  nua_event_t e;
+  nua_client_request_t *cr = ds->ds_cr;
+  struct event_usage *eu = nua_dialog_usage_private(du);
   msg_t *msg;
-  sip_t *sip;
 
-  if (cr->cr_msg) {		/* Already doing something  */
-    /* Delay of 5 .. 15 seconds */
-    du->du_refresh = sip_now() + su_randint(5, 15);
-    du->du_pending = refresh_subscribe;
+  assert(eu);
+  
+  if (eu->eu_final_wait) {
+    /* Did not receive NOTIFY for fetch... */
+    sip_event_t const *o = du->du_event;
+    char const *id = o ? o->o_id : NULL;
+
+    SU_DEBUG_3(("nua(%p): fetch event %s%s%s timeouts\n",
+		nh, o ? o->o_type : "(empty)",
+		id ? "; id=" : "", id ? id : ""));
+
+    nua_stack_event(nh->nh_nua, nh,  NULL,
+		    nua_i_notify, 408, "Fetch Timeouts without NOTIFY", 
+		    NUTAG_SUBSTATE(nua_substate_terminated),
+		    SIPTAG_EVENT(o),
+		    TAG_END());
+
+    nua_dialog_usage_remove(nh, ds, du);
+
     return;
   }
 
-  if (now > 0)
-    e = nua_r_subscribe;
-  else
-    e = nua_r_destroy, du->du_terminating = 1;
+  if (du->du_terminating)	/* No need to refresh. */
+    return;
+
+  if (cr->cr_msg) {
+    /* Already doing something, delay 5..15 seconds? */
+    if (cr->cr_usage != du)
+      nua_dialog_usage_refresh_range(du, 5, 15);
+    return;
+  }
 
   cr->cr_msg = msg_copy(du->du_msg);
 
-  {
-    /* Remove initial route */
-    sip_t *sip = sip_object(cr->cr_msg);
-    if (sip->sip_route)
-      msg_header_remove(cr->cr_msg, (msg_pub_t*)sip, (void *)sip->sip_route);
-  }
-
   msg = nua_creq_msg(nua, nh, cr, 1,
 		     SIP_METHOD_SUBSCRIBE,
 		     NUTAG_USE_DIALOG(1),
 		     NUTAG_ADD_CONTACT(1),
-		     TAG_IF(du->du_terminating, 
-			    SIPTAG_EXPIRES_STR("0")),
+		     /* If dialog is established, remove initial route */
+		     TAG_IF(nua_dialog_is_established(nh->nh_ds),
+			    SIPTAG_ROUTE(NONE)),
 		     TAG_END());
 
-  sip = sip_object(msg);
-
   cr->cr_orq = nta_outgoing_mcreate(nua->nua_nta,
 				    process_response_to_subscribe, nh, NULL,
 				    msg,
 				    SIPTAG_END(), TAG_NEXT(NULL));
-
-  if (!cr->cr_orq) {
-    struct event_usage *eu = nua_dialog_usage_private(du);
-
-    if (du->du_terminating)
-      nua_dialog_usage_remove(nh, nh->nh_ds, du);
-    msg_destroy(msg);
-    UA_EVENT3(e, NUA_INTERNAL_ERROR, 
-	      NUTAG_SUBSTATE(eu->eu_substate), TAG_END());
+  if (cr->cr_orq) {
+    cr->cr_usage = du;
+    cr->cr_event = nua_r_subscribe;
     return;
   }
 
-  cr->cr_usage = du;
-  cr->cr_event = e;
+  if (du->du_terminating)
+    nua_dialog_usage_remove(nh, nh->nh_ds, du);
+  else   /* Try again later? */
+    nua_dialog_usage_refresh_range(du, 5, 15);
+
+  msg_destroy(msg);
+  UA_EVENT3(nua_r_subscribe, NUA_INTERNAL_ERROR, 
+	    NUTAG_SUBSTATE(eu->eu_substate),
+	    TAG_END());
 }
 
 
-/** Terminate subscription when we have not received a NOTIFY */
-static void 
-terminate_subscription(nua_handle_t *nh,
-		       nua_dialog_usage_t *du,
-		       sip_time_t now)
+/** Terminate subscription */
+static int nua_subscribe_usage_shutdown(nua_handle_t *nh,
+					nua_dialog_state_t *ds,
+					nua_dialog_usage_t *du)
 {
-  sip_event_t const *o = NULL;
-  char const *id;
+  nua_t *nua = nh->nh_nua;
+  nua_client_request_t *cr = ds->ds_cr;
+  struct event_usage *eu = nua_dialog_usage_private(du);
+  msg_t *msg;
 
-  if (!du) {
-    SU_DEBUG_1(("nua(%p): terminate_subscription() without usage to remove\n", 
-		nh));
-    return;
+  assert(eu); (void)eu;
+
+  if (du->du_terminating)
+    return 100;			/* ...in progress */
+  
+  if (cr->cr_msg)
+    /* XXX - already doing something else? */
+    return 100;
+
+  cr->cr_msg = msg_copy(du->du_msg);
+
+  msg = nua_creq_msg(nua, nh, cr, 1,
+		     SIP_METHOD_SUBSCRIBE,
+		     NUTAG_USE_DIALOG(1),
+		     NUTAG_ADD_CONTACT(1),
+		     SIPTAG_EXPIRES_STR("0"),
+		     /* If dialog is established, remove initial route */
+		     TAG_IF(nua_dialog_is_established(nh->nh_ds),
+			    SIPTAG_ROUTE(NONE)),
+		     TAG_END());
+
+  cr->cr_orq = nta_outgoing_mcreate(nua->nua_nta,
+				    process_response_to_subscribe, nh, NULL,
+				    msg,
+				    SIPTAG_END(), TAG_NEXT(NULL));
+  if (cr->cr_orq) {
+    cr->cr_usage = du;
+    cr->cr_event = nua_r_destroy;
+    return 100;
   }
 
-  o = du->du_event;
-  id = o ? o->o_id : NULL;
-  SU_DEBUG_3(("nua(%p): terminate_subscription() with event %s%s%s\n",
-	      nh, o ? o->o_type : "(empty)",
-	      id ? "; id=" : "", id ? id : ""));
-
-  nua_stack_event(nh->nh_nua, nh,  NULL,
-	   nua_i_notify, 408, "Early Subscription Timeouts without NOTIFY", 
-	   NUTAG_SUBSTATE(nua_substate_terminated),
-	   SIPTAG_EVENT(o),
-	   TAG_END());
-
-  nua_dialog_usage_remove(nh, nh->nh_ds, du);
+  /* Too bad. */
+  nua_dialog_usage_remove(nh, ds, du);
+  msg_destroy(msg);
+  return 200;
 }
 
-/** @internal Process incoming SUBSCRIBE. */
-int nua_stack_process_subsribe(nua_t *nua,
-			       nua_handle_t *nh,
-			       nta_incoming_t *irq,
-			       sip_t const *sip)
-{
-  nua_dialog_state_t *ds;
-  nua_dialog_usage_t *du = NULL;
-  struct event_usage *eu;
-  sip_event_t *o = sip->sip_event;
-  sip_contact_t const *m = NULL;
-  int status; char const *phrase;
-  unsigned long expires, refer_expires;
-  nua_registration_t *nr;
-
-  enter;
-
-  if (nh)
-    du = nua_dialog_usage_get(ds = nh->nh_ds, nua_notify_usage, o);
-
-  if (nh == NULL || du == NULL) {
-    /* Hard-coded support only for refer subscriptions */
-    if (o && str0cmp(o->o_type, "refer") == 0)
-      nta_incoming_treply(irq, 
-			  SET_STATUS(481, "Subscription Does Not Exist"), 
-			  TAG_END());
-    else
-      nta_incoming_treply(irq,
-			  SET_STATUS1(SIP_489_BAD_EVENT), 
-			  SIPTAG_ALLOW_EVENTS_STR("refer"),
-			  SIPTAG_ACCEPT_STR("message/sipfrag"),
-			  TAG_END());
-
-    nua_stack_event(nua, nh, nta_incoming_getrequest(irq),
-	     nua_i_subscribe, status, phrase, 
-	     NUTAG_SUBSTATE(nua_substate_terminated),
-	     TAG_END());
-
-    return status;
-  }
-
-  /* Refresh existing subscription */
-  eu = nua_dialog_usage_private(du);  assert(nh && du && eu);
-
-  nua_dialog_store_peer_info(nh, nh->nh_ds, sip);
-  nua_dialog_uas_route(nh, nh->nh_ds, sip, 1);
-
-  refer_expires = NH_PGET(nh, refer_expires);
-  expires = refer_expires;
-
-  if (sip->sip_expires) {
-    expires = sip->sip_expires->ex_delta;
-    if (expires > refer_expires)
-      expires = refer_expires;
-  }
-
-  if (expires == 0) {
-    eu->eu_substate = nua_substate_terminated;
-    du->du_refresh = sip_now();
-  }
-  else {
-    du->du_refresh = sip_now() + expires;
-  }
-
-  if (eu->eu_substate == nua_substate_pending)
-    SET_STATUS1(SIP_202_ACCEPTED);
-  else
-    SET_STATUS1(SIP_200_OK);
-
-  if (status < 300) {
-    nr = nua_registration_by_aor(nua->nua_registrations,
-				 sip->sip_to,
-				 sip->sip_request->rq_url,
-				 0);
-    m = nua_registration_contact(nr);
-    if (m == NULL) {
-      SET_STATUS1(SIP_500_INTERNAL_SERVER_ERROR); 
-      eu->eu_substate = nua_substate_terminated;
-    }
-  }
-    
-  nta_incoming_treply(irq, status, phrase, SIPTAG_CONTACT(m), NULL);
-
-  nua_stack_event(nua, nh, nta_incoming_getrequest(irq),
-	   nua_i_subscribe, status, phrase, 
-	   NUTAG_SUBSTATE(eu->eu_substate),
-	   TAG_END());
-
-  nta_incoming_destroy(irq), irq = NULL;
-
-  /* Immediate notify */
-  if (status < 300)
-    nua_stack_post_signal(nh, nua_r_notify,
-			  SIPTAG_EVENT(du->du_event),
-			  TAG_END());
-
-  return 0;
-}
-
-
 /* ======================================================================== */
-/* NOTIFY */
+/* NOTIFY server */
 
-static int process_response_to_notify(nua_handle_t *nh,
-				      nta_outgoing_t *orq,
-				      sip_t const *sip);
-
-int
-nua_stack_notify(nua_t *nua, nua_handle_t *nh, nua_event_t e, tagi_t const *tags)
-{
-  struct nua_client_request *cr = nh->nh_cr;
-  nua_dialog_usage_t *du = NULL;
-  struct event_usage *eu;
-  msg_t *msg;
-  sip_t *sip;
-
-  if (cr->cr_orq) {
-    return UA_EVENT2(e, 500, "Request already in progress");
-  }
-
-  nua_stack_init_handle(nua, nh, nh_has_nothing, NULL, TAG_NEXT(tags));
-
-  msg = nua_creq_msg(nua, nh, cr, cr->cr_retry_count,
-			 SIP_METHOD_NOTIFY,
-			 NUTAG_ADD_CONTACT(1),
-			 TAG_NEXT(tags));
-
-  sip = sip_object(msg);
-  if (!sip)
-    return UA_EVENT1(e, NUA_INTERNAL_ERROR);
-
-  du = nua_dialog_usage_get(nh->nh_ds, nua_notify_usage, sip->sip_event);
-  eu = nua_dialog_usage_private(du);
-
-  if (du && du->du_event && !sip->sip_event)
-    sip_add_dup(msg, sip, (sip_header_t *)du->du_event);
-
-  if (!du)
-    ;
-  else if (sip->sip_subscription_state) {
-    char const *ss_substate = sip->sip_subscription_state->ss_substate;
-    sip_time_t now = sip_now();
-
-    if (strcasecmp(ss_substate, "terminated") == 0)
-      eu->eu_substate = nua_substate_terminated;
-    else if (strcasecmp(ss_substate, "pending") == 0)
-      eu->eu_substate = nua_substate_pending;
-    else /* if (strcasecmp(subs->ss_substate, "active") == 0) */ 
-      eu->eu_substate = nua_substate_active;
-
-    if (sip->sip_subscription_state->ss_expires) {
-      unsigned long expires;
-      expires = strtoul(sip->sip_subscription_state->ss_expires, NULL, 10);
-      if (expires > 3600)
-        expires = 3600;
-      du->du_refresh = now + expires;
-    }
-    else if (eu->eu_substate != nua_substate_terminated) {
-      sip_subscription_state_t *ss = sip->sip_subscription_state;
-      char *param;
-
-      if (now < du->du_refresh)
-        param = su_sprintf(msg_home(msg), "expires=%lu", 
-      		     du->du_refresh - now);
-      else
-        param = "expires=0";
-
-      msg_header_add_param(msg_home(msg), ss->ss_common, param);
-    }
-  }
-  else {
-    sip_subscription_state_t *ss;
-    char const *substate;
-    sip_time_t now = sip_now();
-    unsigned long expires = 3600;
-
-    switch (eu->eu_substate) {
-    case nua_substate_embryonic:
-      eu->eu_substate = nua_substate_pending;
-      /*FALLTHROUGH*/
-    case nua_substate_pending:
-      substate = "pending";
-      break;
-    case nua_substate_active:
-    default:
-      substate = "active";
-      break;
-    case nua_substate_terminated:
-      substate = "terminated";
-      break;
-    }
-
-    if (du->du_refresh <= now)
-      eu->eu_substate = nua_substate_terminated;
-
-    if (eu->eu_substate != nua_substate_terminated) {
-      if (du->du_refresh)
-        expires = du->du_refresh - now;
-      else
-        du->du_refresh = now + expires;
-      ss = sip_subscription_state_format(msg_home(msg), "%s;expires=%lu",
-      				   substate, expires);
-    }
-    else {
-      du->du_refresh = now;
-      ss = sip_subscription_state_make(msg_home(msg), "terminated; "
-      				 "reason=noresource");
-    }
-
-    msg_header_insert(msg, (void *)sip, (void *)ss);
-  }
-
-  if (du) {
-    if (eu->eu_substate == nua_substate_terminated)
-      du->du_terminating = 1;
-    cr->cr_orq = nta_outgoing_mcreate(nua->nua_nta,
-				      process_response_to_notify, nh, NULL,
-				      msg,
-				      SIPTAG_END(), TAG_NEXT(tags));
-  }
-
-  if (!cr->cr_orq) {
-    msg_destroy(msg);
-    return UA_EVENT1(e, NUA_INTERNAL_ERROR);
-  }
-
-  cr->cr_usage = du;
-
-  return cr->cr_event = e;
-}
-
-static
-void restart_notify(nua_handle_t *nh, tagi_t *tags)
-{
-  nua_creq_restart(nh, nh->nh_cr, process_response_to_notify, tags);
-}
-
-static int process_response_to_notify(nua_handle_t *nh,
-				      nta_outgoing_t *orq,
-				      sip_t const *sip)
-{
-  if (nua_creq_check_restart(nh, nh->nh_cr, orq, sip, restart_notify))
-    return 0;
-  return nua_stack_process_response(nh, nh->nh_cr, orq, sip, TAG_END());
-}
+/** @NUA_EVENT nua_i_notify
+ *
+ * Event for incoming NOTIFY request.
+ *
+ * @param status statuscode of response sent automatically by stack
+ * @param phrase a short textual description of @a status code
+ * @param nh     operation handle associated with the subscription
+ * @param hmagic application context associated with the handle
+ * @param sip    incoming NOTIFY request
+ * @param tags   NUTAG_SUBSTATE() indicating the subscription state
+ *
+ * @sa nua_subscribe(), nua_unsubscribe(), @RFC3265, #nua_i_subscribe
+ * 
+ * @END_NUA_EVENT
+ */
 
 /** @internal Process incoming NOTIFY. */
 int nua_stack_process_notify(nua_t *nua,
@@ -713,7 +611,8 @@ int nua_stack_process_notify(nua_t *nua,
   assert(nh);
 
   if (/* XXX - support forking of subscriptions?... */
-      ds->ds_remote_tag && sip && sip->sip_from->a_tag &&
+      ds->ds_remote_tag && ds->ds_remote_tag[0] && 
+      sip && sip->sip_from->a_tag &&
       strcmp(ds->ds_remote_tag, sip->sip_from->a_tag)) {
     sip_contact_t const *m = NULL;
     sip_warning_t *w = NULL, w0[1];
@@ -741,24 +640,26 @@ int nua_stack_process_notify(nua_t *nua,
   }
 
   eu = nua_dialog_usage_private(du); assert(eu);
+  eu->eu_notified++;
+
+  if (!sip->sip_event->o_id) {
+    eu->eu_no_id = 1;
+  }
 
   if (subs == NULL) {
     /* Do some compatibility stuff here */
-    unsigned long delta = 3600;
+    unsigned long delta;
 
     sip_subscription_state_init(subs = ss0);
 
-    if (sip->sip_expires)
-      delta = sip->sip_expires->ex_delta;
-    else
-      delta = eu->eu_expires;
+    delta = sip->sip_expires ? sip->sip_expires->ex_delta : eu->eu_expires;
 
     if (delta == 0)
       subs->ss_substate = "terminated";
     else
       subs->ss_substate = "active";
 
-    if (delta > 0) {
+    if (delta > 0 && sip->sip_expires) {
       snprintf(expires, sizeof expires, "%lu", delta);
       subs->ss_expires = expires;
     }
@@ -772,13 +673,15 @@ int nua_stack_process_notify(nua_t *nua,
 
     if (str0casecmp(subs->ss_reason, why = "deactivated") == 0) {
       eu->eu_substate = nua_substate_embryonic;
-      retry = 0;
+      retry = 0;		/* retry immediately */
     } 
     else if (str0casecmp(subs->ss_reason, why = "probation") == 0) {
       eu->eu_substate = nua_substate_embryonic;
       retry = 30;
       if (subs->ss_retry_after)
 	retry = strtoul(subs->ss_retry_after, NULL, 10);
+      if (retry > 3600)
+	retry = 3600;
     }
     else
       why = subs->ss_reason;
@@ -786,25 +689,34 @@ int nua_stack_process_notify(nua_t *nua,
   else if (strcasecmp(subs->ss_substate, what = "pending") == 0)
     eu->eu_substate = nua_substate_pending;
   else /* if (strcasecmp(subs->ss_substate, "active") == 0) */ {
+    /* Any extended state is considered as active */
     what = subs->ss_substate ? subs->ss_substate : "active";
-    /* XXX - any extended state is considered as active */
     eu->eu_substate = nua_substate_active;
   }
+
+  if (du->du_terminating)
+    retry = -1;
   
   response = nh_make_response(nua, nh, irq, SIP_200_OK,
 			      SIPTAG_ALLOW(NH_PGET(nh, allow)),
 			      SIPTAG_SUPPORTED(NH_PGET(nh, supported)),
 			      TAG_END());
 
-  if (response && nua_registration_add_contact(nh, response, NULL, 1, 0) >= 0)
+  if (response && 
+      nua_registration_add_contact_to_response(nh, response, NULL, 
+					       sip->sip_record_route,
+					       sip->sip_contact) >= 0)
     nta_incoming_mreply(irq, response);
   else
     nta_incoming_treply(irq, SIP_500_INTERNAL_SERVER_ERROR, TAG_END());
 
+  if (eu->eu_substate == nua_substate_terminated && retry > 0)
+    eu->eu_substate = nua_substate_embryonic;
+
   nua_stack_event(nh->nh_nua, nh, nta_incoming_getrequest(irq),
-	   nua_i_notify, SIP_200_OK, 
-	   NUTAG_SUBSTATE(eu->eu_substate),
-	   TAG_END());
+		  nua_i_notify, SIP_200_OK, 
+		  NUTAG_SUBSTATE(eu->eu_substate),
+		  TAG_END());
 
   nta_incoming_destroy(irq), irq = NULL;
 
@@ -812,27 +724,34 @@ int nua_stack_process_notify(nua_t *nua,
 	      nh, what, why ? why : ""));
 
   if (eu->eu_substate == nua_substate_terminated) {
-    du->du_refresh = 0, du->du_pending = NULL;
-    if (du != nh->nh_cr->cr_usage)
+    if (du != nh->nh_ds->ds_cr->cr_usage)
       nua_dialog_usage_remove(nh, nh->nh_ds, du);
+    else
+      nua_dialog_usage_reset_refresh(du);
   }
   else if (eu->eu_substate == nua_substate_embryonic) {
-    if (retry != -1 && !du->du_terminating) {
+    if (retry >= 0) {
       /* Try to subscribe again */
-      /* XXX - how to handle dialog ?? */
-      nua_dialog_usage_set_refresh(du, retry);
-      du->du_pending = refresh_subscribe;
+      nua_dialog_remove(nh, nh->nh_ds, du); /* tear down */
+      nua_dialog_usage_refresh_range(du, retry, retry + 5);
     }
-    else if (du != nh->nh_cr->cr_usage)
+    else if (du != nh->nh_ds->ds_cr->cr_usage)
       nua_dialog_usage_remove(nh, nh->nh_ds, du);
+    else
+      nua_dialog_usage_reset_refresh(du);
   }
-  else if (subs->ss_expires) {
-    sip_time_t delta = strtoul(subs->ss_expires, NULL, 10);
+  else if (du->du_terminating) {
+    nua_dialog_usage_reset_refresh(du);
+  }
+  else {
+    sip_time_t delta;
+
+    if (subs->ss_expires)
+      delta = strtoul(subs->ss_expires, NULL, 10);
+    else
+      delta = eu->eu_expires;
     
-    if (!du->du_terminating) {
-      nua_dialog_usage_set_refresh(du, delta);
-      du->du_pending = refresh_subscribe;
-    }
+    nua_dialog_usage_set_refresh(du, delta);
   }
 
   return 0;
@@ -841,6 +760,48 @@ int nua_stack_process_notify(nua_t *nua,
 /* ======================================================================== */
 /* REFER */
 
+/** Transfer a call. 
+ * 
+ * Send a REFER request asking the recipient to transfer the call. 
+ *
+ * The REFER request also establishes an implied subscription to the "refer"
+ * event. The "refer" event can have an "id" parameter, which has the value
+ * of CSeq number in the REFER request. After initiating the REFER request,
+ * the nua engine sends application a #nua_r_refer event with status 100 and
+ * tag NUTAG_REFER_EVENT() containing a matching event header with id
+ * parameter.
+ *
+ * Note that the @Event header in the locally generated #nua_r_refer event
+ * contains the @a id parameter. The @a id parameter contains the @CSeq
+ * number of the REFER request, and it may get incremented if the request is
+ * retried because it got challenged or redirected. In that case, the
+ * application gets a new #nua_r_refer event with status 100 and tag
+ * NUTAG_REFER_EVENT(). Also the recipient of the REFER request may or may
+ * not include the @a id parameter with the @Event header in the NOTIFY
+ * requests messages which it sends to the sender of the REFER request.
+ *
+ * Therefore the application is not able to modify the state of the implied
+ * subscription before receiving the first NOTIFY request.
+ *
+ * @param nh              Pointer to operation handle
+ * @param tag, value, ... List of tagged parameters
+ *
+ * @return 
+ *    nothing
+ *
+ * @par Related Tags:
+ *    NUTAG_URL() \n
+ *    Tags of nua_set_hparams() \n
+ *    Tags in <sip_tag.h>
+ *
+ * @par Events:
+ *    #nua_r_refer \n
+ *    #nua_i_notify
+ *
+ * @sa #nua_r_refer, NUTAG_SUBSTATE(), NUTAG_REFER_EVENT(),#nua_i_refer,
+ * @RFC3515, @ReferTo, @RFC3892, @ReferredBy
+ */
+
 static int process_response_to_refer(nua_handle_t *nh,
 				     nta_outgoing_t *orq,
 				     sip_t const *sip);
@@ -849,22 +810,18 @@ int
 nua_stack_refer(nua_t *nua, nua_handle_t *nh, nua_event_t e, tagi_t const *tags)
 {
   nua_dialog_usage_t *du = NULL;
-  struct nua_client_request *cr = nh->nh_cr;
+  nua_client_request_t *cr = nh->nh_ds->ds_cr;
   msg_t *msg;
   sip_t *sip;
   sip_referred_by_t by[1];
   sip_event_t *event = NULL;
 
-  if (nh_is_special(nh) && !nua_handle_has_subscribe(nh)) {
-    return UA_EVENT2(e, 500, "Invalid handle for REFER");
-  }
-  else if (cr->cr_orq) {
-    return UA_EVENT2(e, 500, "Request already in progress");
-  }
+  if (nua_stack_set_handle_special(nh, nh_has_subscribe, nua_r_subscribe) < 0)
+    return UA_EVENT2(e, 900, "Invalid handle for REFER");
+  else if (cr->cr_orq)
+    return UA_EVENT2(e, 900, "Request already in progress");
 
-  nua_stack_init_handle(nua, nh, nh_has_subscribe, "NOTIFY", TAG_NEXT(tags));
-  if (nh->nh_has_subscribe)
-    nh->nh_special = nua_r_subscribe;
+  nua_stack_init_handle(nua, nh, TAG_NEXT(tags));
 
   sip_referred_by_init(by);
   by->b_display = nua->nua_from->a_display;
@@ -917,14 +874,37 @@ nua_stack_refer(nua_t *nua, nua_handle_t *nh, nua_event_t e, tagi_t const *tags)
 
 void restart_refer(nua_handle_t *nh, tagi_t *tags)
 {
-  nua_stack_refer(nh->nh_nua, nh, nh->nh_cr->cr_event, tags);
+  nua_stack_refer(nh->nh_nua, nh, nh->nh_ds->ds_cr->cr_event, tags);
 }
 
+/**@NUA_EVENT nua_r_refer
+ *
+ * @brief Response to outgoing REFER.
+ *
+ * @param status response status code
+ *               (if the request is retried, @a status is 100, the @a
+ *               sip->sip_status->st_status contain the real status code
+ *               from the response message, e.g., 302, 401, or 407)
+ * @param phrase a short textual description of @a status code
+ * @param nh     operation handle associated with the REFER request
+ * @param hmagic application context associated with the handle
+ * @param sip    response to REFER request or NULL upon an error
+ *               (status code is in @a status and 
+ *                descriptive message in @a phrase parameters)
+ * @param tags    NUTAG_REFER_EVENT() \n
+ *                NUTAG_SUBSTATE()
+ *
+ * @sa nua_refer(), NUTAG_SUBSTATE(), #nua_i_refer,
+ * @RFC3515, @ReferTo, @RFC3892, @ReferredBy
+ *
+ * @END_NUA_EVENT
+ */
+
 static int process_response_to_refer(nua_handle_t *nh,
 				     nta_outgoing_t *orq,
 				     sip_t const *sip)
 {
-  struct nua_client_request *cr = nh->nh_cr;
+  nua_client_request_t *cr = nh->nh_ds->ds_cr;
   int status = sip ? sip->sip_status->st_status : 408;
 
   if (status < 200)
@@ -944,76 +924,3 @@ static int process_response_to_refer(nua_handle_t *nh,
 
   return nua_stack_process_response(nh, cr, orq, sip, TAG_END());
 }
-
-/** @internal Process incoming REFER. */
-int nua_stack_process_refer(nua_t *nua,
-			    nua_handle_t *nh,
-			    nta_incoming_t *irq,
-			    sip_t const *sip)
-{
-  nua_dialog_usage_t *du = NULL;
-  sip_event_t *event;
-  sip_referred_by_t *by = NULL, default_by[1];
-  msg_t *response;
-  int created = 0;
-
-  if (nh == NULL) {
-    if (!(nh = nua_stack_incoming_handle(nua, irq, sip, nh_has_notify, 1)))
-      return 500;
-    created = 1;
-  }
-
-  event = sip_event_format(nh->nh_home, "refer;id=%u", sip->sip_cseq->cs_seq);
-  if (event)
-    du = nua_dialog_usage_add(nh, nh->nh_ds, nua_notify_usage, event);
-
-  if (!du || du->du_ready) {
-    if (du->du_ready) {
-      SU_DEBUG_1(("nua(%p): REFER with existing refer;id=%u\n", nh,
-		  sip->sip_cseq->cs_seq));
-    }
-    if (created) 
-      nh_destroy(nua, nh);
-    return 500;
-  }
-
-  du->du_ready = 1;
-
-  nua_dialog_uas_route(nh, nh->nh_ds, sip, 1);	/* Set route and tags */
-
-  if (!sip->sip_referred_by) {
-    sip_from_t *a = sip->sip_from;
-
-    sip_referred_by_init(by = default_by);
-
-    *by->b_url = *a->a_url;
-    by->b_display = a->a_display;
-  }
-
-  response = nh_make_response(nua, nh, irq, 
-			      SIP_202_ACCEPTED, 
-			      NUTAG_ADD_CONTACT(1),
-			      TAG_END());
-
-  nta_incoming_mreply(irq, response);
-
-  du->du_refresh = sip_now() + NH_PGET(nh, refer_expires);
-
-  /* Immediate notify */
-  nua_stack_post_signal(nh,
-			nua_r_notify,
-			SIPTAG_EVENT(event),
-			SIPTAG_CONTENT_TYPE_STR("message/sipfrag"),
-			SIPTAG_PAYLOAD_STR("SIP/2.0 100 Trying\r\n"),
-			TAG_END());
-  
-  nua_stack_event(nh->nh_nua, nh, nta_incoming_getrequest(irq),
-		  nua_i_refer, SIP_202_ACCEPTED, 
-		  NUTAG_REFER_EVENT(event),
-		  TAG_IF(by, SIPTAG_REFERRED_BY(by)),
-		  TAG_END());
-  
-  su_free(nh->nh_home, event);
-
-  return 500;   
-}
