@@ -105,7 +105,7 @@ struct nat {
   /* ...but source address will be "fake" */
   su_localinfo_t *localinfo, *private, *fake;
 
-  int udp_socket, tcp_socket;
+  su_socket_t udp_socket, tcp_socket;
   int udp_register, tcp_register;
 
   char buffer[65536];
@@ -118,7 +118,7 @@ struct binding
   struct binding *next, **prev;
   struct nat *nat;		/* backpointer */
   int socktype, protocol;
-  int in_socket, out_socket;
+  su_socket_t in_socket, out_socket;
   int in_register, out_register;
   int in_closed, out_closed;
   char in_name[64], out_name[64];
@@ -128,7 +128,7 @@ static struct binding *nat_binding_new(struct nat *nat,
 				       char const *protoname,
 				       int socktype, int protocol, 
 				       int connected,
-				       int in_socket,
+				       su_socket_t in_socket,
 				       su_sockaddr_t *from,
 				       socklen_t fromlen);
 static void nat_binding_destroy(struct binding *);
@@ -141,7 +141,7 @@ static int binding_init(struct binding *b,
 			socklen_t fromlen);
 
 static void flush_bindings(struct nat *nat);
-static void invalidate_bindings(struct nat *nat);
+static int invalidate_bindings(void *nat);
 
 static int new_udp(struct nat *, su_wait_t *wait, struct binding *dummy);
 static int udp_in_to_out(struct nat *, su_wait_t *wait, struct binding *);
@@ -165,7 +165,7 @@ test_nat_init(su_root_t *root, struct nat *nat)
   su_wait_t wait[1];
 
   nat->root = root;
-  nat->udp_socket = -1, nat->tcp_socket = -1;
+  nat->udp_socket = INVALID_SOCKET, nat->tcp_socket = INVALID_SOCKET;
 
   tl_gets(nat->tags, 
 	  TESTNATTAG_SYMMETRIC_REF(nat->symmetric),
@@ -201,7 +201,7 @@ test_nat_init(su_root_t *root, struct nat *nat)
   /* Bind TCP and UDP to same port */
   for (;;) {
     nat->udp_socket = su_socket(li->li_family, SOCK_DGRAM, IPPROTO_UDP);
-    if (nat->udp_socket == -1)
+    if (nat->udp_socket == INVALID_SOCKET)
       return -1;
 
     if (bind(nat->udp_socket, (void *)su, sulen) < 0) {
@@ -212,9 +212,9 @@ test_nat_init(su_root_t *root, struct nat *nat)
 
       fprintf(stderr, "test_nat: port %u: %s\n",
 	      port, su_strerror(su_errno()));
-
       su_close(nat->udp_socket);
-      nat->udp_socket = -1;
+
+      nat->udp_socket = INVALID_SOCKET;
 
       if (++port > 65535)
 	port = 1024;
@@ -240,12 +240,12 @@ test_nat_init(su_root_t *root, struct nat *nat)
     }
 
     nat->tcp_socket = su_socket(li->li_family, SOCK_STREAM, IPPROTO_TCP);
-    if (nat->tcp_socket == -1)
+    if (nat->tcp_socket == INVALID_SOCKET)
       return -1;
 
     if (bind(nat->tcp_socket, (void *)su, sulen) < 0) {
       su_close(nat->tcp_socket);
-      nat->tcp_socket = -1;
+      nat->tcp_socket = INVALID_SOCKET;
 
       fprintf(stderr, "test_nat: port %u: %s\n",
 	      port, su_strerror(su_errno()));
@@ -311,9 +311,9 @@ test_nat_deinit(su_root_t *root, struct nat *nat)
   if (nat->udp_register)
     su_root_deregister(root, nat->udp_register);
 
-  if (nat->udp_socket != -1)
+  if (nat->udp_socket != INVALID_SOCKET)
     su_close(nat->udp_socket);
-  if (nat->tcp_socket != -1)
+  if (nat->tcp_socket != INVALID_SOCKET)
     su_close(nat->tcp_socket);
 
   su_freelocalinfo(nat->localinfo);
@@ -417,7 +417,7 @@ int test_nat_flush(struct nat *nat)
     return su_seterrno(EFAULT);
 
   return su_task_execute(su_clone_task(nat->clone), 
-			 (void *)invalidate_bindings, nat, NULL);
+			 invalidate_bindings, nat, NULL);
 }
 
 /* ====================================================================== */
@@ -427,7 +427,7 @@ struct binding *nat_binding_new(struct nat *nat,
 				int socktype,
 				int protocol,
 				int connected,
-				int in_socket,
+				su_socket_t in_socket,
 				su_sockaddr_t *from,
 				socklen_t fromlen)
 {
@@ -449,7 +449,7 @@ struct binding *nat_binding_new(struct nat *nat,
   b->nat = nat;
   b->socktype = socktype;
   b->protocol = protocol;
-  b->in_socket = in_socket, b->out_socket = -1;
+  b->in_socket = in_socket, b->out_socket = INVALID_SOCKET;
   b->in_register = -1, b->out_register = -1;
 
   if (binding_init(b, protoname, connected, nat->fake, from, fromlen) < 0)
@@ -466,7 +466,7 @@ static int binding_init(struct binding *b,
 			socklen_t fromlen)
 {
   struct nat *nat = b->nat;
-  int out_socket;
+  su_socket_t out_socket;
   su_sockaddr_t addr[1];
   socklen_t addrlen = (sizeof addr);
   char ipname[64];
@@ -478,11 +478,11 @@ static int binding_init(struct binding *b,
   else
     in_to_out = udp_in_to_out, out_to_in = udp_out_to_in;    
   
-  if (b->in_socket == -1) {
+  if (b->in_socket == INVALID_SOCKET) {
     int in_socket;
 
     in_socket = su_socket(from->su_family, b->socktype, b->protocol);
-    if (in_socket < 0) {
+    if (in_socket == INVALID_SOCKET) {
       su_perror("nat_binding_new: socket");
       return -1;
     }
@@ -502,7 +502,7 @@ static int binding_init(struct binding *b,
   }
 
   out_socket = su_socket(li->li_family, b->socktype, b->protocol);
-  if (out_socket < 0) {
+  if (out_socket == INVALID_SOCKET) {
     su_perror("nat_binding_new: socket");
     return -1;
   }
@@ -586,13 +586,15 @@ static void flush_bindings(struct nat *nat)
   }
 }
 
-static void invalidate_bindings(struct nat *nat)
+static int invalidate_bindings(void *arg)
 {
+  struct nat *nat = arg;
   struct binding *b;
 
   for (b = nat->bindings; b; b = b->next) {
     invalidate_binding(b);
   }
+  return 0;
 }
 
 #if 0
@@ -617,7 +619,7 @@ static struct binding *nat_binding_find(struct nat *nat,
 
   if (b == NULL)
     b = nat_binding_new(nat, "UDP", SOCK_DGRAM, IPPROTO_UDP, nat->symmetric, 
-			-1, from, fromlen);
+			INVALID_SOCKET, from, fromlen);
 
   return b;
 }
@@ -639,23 +641,23 @@ static int new_udp(struct nat *nat, su_wait_t *wait, struct binding *dummy)
 
   events = su_wait_events(wait, nat->udp_socket);
 
-  n = recvfrom(nat->udp_socket, nat->buffer, sizeof nat->buffer, 0,
-	       (void *)from, &fromlen);
+  n = su_recvfrom(nat->udp_socket, nat->buffer, sizeof nat->buffer, 0,
+		  from, &fromlen);
   if (n < 0) {
     su_perror("new_udp: recvfrom");
     return 0;
   }
 
   b = nat_binding_new(nat, "UDP", SOCK_DGRAM, IPPROTO_UDP, nat->symmetric, 
-		      -1, from, fromlen);
+		      INVALID_SOCKET, from, fromlen);
   if (b == NULL)
     return 0;
 
   if (nat->symmetric)
-    m = send(b->out_socket, nat->buffer, n, 0);
+    m = su_send(b->out_socket, nat->buffer, n, 0);
   else
-    m = sendto(b->out_socket, nat->buffer, n, 0, 
-	       (void *)nat->out_address, nat->out_addrlen);
+    m = su_sendto(b->out_socket, nat->buffer, n, 0, 
+		  nat->out_address, nat->out_addrlen);
 
   if (nat->logging)
     printf("nat: udp out %d/%d %s => %s\n",
@@ -671,17 +673,17 @@ static int udp_in_to_out(struct nat *nat, su_wait_t *wait, struct binding *b)
 
   events = su_wait_events(wait, b->in_socket);
 
-  n = recv(b->in_socket, nat->buffer, sizeof nat->buffer, 0);
+  n = su_recv(b->in_socket, nat->buffer, sizeof nat->buffer, 0);
   if (n < 0) {
     su_perror("udp_in_to_out: recv");
     return 0;
   }
 
   if (nat->symmetric)
-    m = send(b->out_socket, nat->buffer, n, 0);
+    m = su_send(b->out_socket, nat->buffer, n, 0);
   else
-    m = sendto(b->out_socket, nat->buffer, n, 0,
-	       (void *)nat->out_address, nat->out_addrlen);
+    m = su_sendto(b->out_socket, nat->buffer, n, 0,
+		  nat->out_address, nat->out_addrlen);
 
   if (nat->logging)
     printf("nat: udp out %d/%d %s => %s\n",
@@ -697,13 +699,13 @@ static int udp_out_to_in(struct nat *nat, su_wait_t *wait, struct binding *b)
 
   events = su_wait_events(wait, b->out_socket);
 
-  n = recv(b->out_socket, nat->buffer, sizeof nat->buffer, 0);
+  n = su_recv(b->out_socket, nat->buffer, sizeof nat->buffer, 0);
   if (n < 0) {
     su_perror("udp_out_to_out: recv");
     return 0;
   }
 
-  m = send(b->in_socket, nat->buffer, n, 0);
+  m = su_send(b->in_socket, nat->buffer, n, 0);
 
   if (nat->logging)
     printf("nat: udp in %d/%d %s => %s\n",
@@ -717,7 +719,7 @@ static int udp_out_to_in(struct nat *nat, su_wait_t *wait, struct binding *b)
 static int new_tcp(struct nat *nat, su_wait_t *wait, struct binding *dummy)
 {
   int events;
-  int in_socket;
+  su_socket_t in_socket;
   su_sockaddr_t from[1];
   socklen_t fromlen = (sizeof from);
   struct binding *b;
@@ -725,7 +727,7 @@ static int new_tcp(struct nat *nat, su_wait_t *wait, struct binding *dummy)
   events = su_wait_events(wait, nat->tcp_socket);
 
   in_socket = accept(nat->tcp_socket, (void *)from, &fromlen);
-  if (in_socket < 0) {
+  if (in_socket == INVALID_SOCKET) {
     su_perror("new_tcp: accept");
     return 0;
   }
@@ -745,7 +747,7 @@ static int tcp_in_to_out(struct nat *nat, su_wait_t *wait, struct binding *b)
 
   events = su_wait_events(wait, b->in_socket);
 
-  n = recv(b->in_socket, nat->buffer, sizeof nat->buffer, 0);
+  n = su_recv(b->in_socket, nat->buffer, sizeof nat->buffer, 0);
   if (n < 0) {
     su_perror("tcp_in_to_out: recv");
     return 0;
@@ -763,7 +765,7 @@ static int tcp_in_to_out(struct nat *nat, su_wait_t *wait, struct binding *b)
   }
 
   for (m = 0; m < n; m += o) {
-    o = send(b->out_socket, nat->buffer + m, n - m, 0);
+    o = su_send(b->out_socket, nat->buffer + m, n - m, 0);
     if (o < 0) {
       su_perror("tcp_in_to_out: send");
       break;
@@ -784,7 +786,7 @@ static int tcp_out_to_in(struct nat *nat, su_wait_t *wait, struct binding *b)
 
   events = su_wait_events(wait, b->out_socket);
 
-  n = recv(b->out_socket, nat->buffer, sizeof nat->buffer, 0);
+  n = su_recv(b->out_socket, nat->buffer, sizeof nat->buffer, 0);
   if (n < 0) {
     su_perror("tcp_out_to_in: recv");
     return 0;
@@ -802,7 +804,7 @@ static int tcp_out_to_in(struct nat *nat, su_wait_t *wait, struct binding *b)
   }
 
   for (m = 0; m < n; m += o) {
-    o = send(b->in_socket, nat->buffer + m, n - m, 0);
+    o = su_send(b->in_socket, nat->buffer + m, n - m, 0);
     if (o < 0) {
       if (su_errno() != EPIPE)
 	su_perror("tcp_in_to_out: send");
@@ -822,12 +824,13 @@ static int invalidate_binding(struct binding *b)
   struct nat *nat = b->nat;
   su_sockaddr_t addr[1];
   socklen_t addrlen = (sizeof addr);
-  int out, out_register;
+  su_socket_t out;
+  int out_register;
   su_wait_t wout[1];
   char name[64];
 
   out = su_socket(nat->fake->li_family, b->socktype, 0);
-  if (out < 0) {
+  if (out == INVALID_SOCKET) {
     su_perror("new_udp: socket");
     return -1;
   }
