@@ -69,7 +69,7 @@
  *
  * The Authorization header consists of credentials containing the
  * authentication information of the user agent for the realm of the
- * resource being requested.  Its syntax is defined in [RFC2617, S19]
+ * resource being requested. Its syntax is defined in @RFC2617 and @RFC3261
  * as follows:
  *
  * @code
@@ -101,17 +101,40 @@
  *    auth-scheme       =  token
  * @endcode
  *
+ * The parsed Authorization header
+ * is stored in #sip_authorization_t structure.
+ *
+ * @sa @RFC2617, auth_mod_verify(), auth_mod_check(), auth_get_params(),
+ * auth_digest_response_get().
+ */
+
+/**@ingroup sip_authorization
+ * @typedef typedef struct sip_authorization_s sip_authorization_t;
+ *
+ * The structure #sip_authorization_t contains representation of SIP
+ * @Authorization header.
+ *
+ * The #sip_authorization_t is defined as follows:
+ * @code
+ * typedef struct msg_auth_s {
+ *   msg_common_t       au_common[1];  // Common fragment info 
+ *   msg_auth_t        *au_next;       // Link to next header 
+ *   char const        *au_scheme;     // Auth-scheme like "Basic" or "Digest" 
+ *   msg_param_t const *au_params;     // Comma-separated parameters
+ * } sip_authorization_t;
+ * @endcode
+ *
  */
 
 msg_hclass_t sip_authorization_class[] =
 SIP_HEADER_CLASS_AUTH(authorization, "Authorization", single);
 
-int sip_authorization_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_authorization_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
   return msg_auth_d(home, h, s, slen);
 }
 
-int sip_authorization_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_authorization_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   assert(sip_is_authorization(h));
   return msg_auth_e(b, bsiz, h, f);
@@ -150,20 +173,39 @@ int sip_authorization_e(char b[], int bsiz, sip_header_t const *h, int f)
  *    qop-value           =  "auth" / "auth-int" / token
  * @endcode
  *
- * @todo Currently the parser fails if several challenges are included in
- * one header.
+ *
+ * The parsed Proxy-Authenticate header
+ * is stored in #sip_proxy_authenticate_t structure.
+ */
+
+/**@ingroup sip_proxy_authenticate
+ * @typedef typedef struct sip_proxy_authenticate_s sip_proxy_authenticate_t;
+ *
+ * The structure #sip_proxy_authenticate_t contains representation of SIP
+ * @ProxyAuthenticate header.
+ *
+ * The #sip_proxy_authenticate_t is defined as follows:
+ * @code
+ * typedef struct msg_auth_s {
+ *   msg_common_t       au_common[1];  // Common fragment info 
+ *   msg_auth_t        *au_next;       // Link to next header 
+ *   char const        *au_scheme;     // Auth-scheme like "Basic" or "Digest" 
+ *   msg_param_t const *au_params;     // Comma-separated parameters
+ * } sip_proxy_authenticate_t;
+ * @endcode
+ *
  */
 
 msg_hclass_t sip_proxy_authenticate_class[] =
 SIP_HEADER_CLASS_AUTH(proxy_authenticate, "Proxy-Authenticate", append);
 
-int sip_proxy_authenticate_d(su_home_t *home,
-			     sip_header_t *h, char *s, int slen)
+issize_t sip_proxy_authenticate_d(su_home_t *home, sip_header_t *h,
+				  char *s, isize_t slen)
 {
   return msg_auth_d(home, h, s, slen);
 }
 
-int sip_proxy_authenticate_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_proxy_authenticate_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   assert(sip_is_proxy_authenticate(h));
   return msg_auth_e(b, bsiz, h, f);
@@ -185,18 +227,41 @@ int sip_proxy_authenticate_e(char b[], int bsiz, sip_header_t const *h, int f)
  *                            / other-response
  * @endcode
  *
+ * @sa auth_mod_verify(), auth_mod_check(), auth_get_params(),
+ * auth_digest_response_get().
+ *
+ * The parsed Proxy-Authorization header
+ * is stored in #sip_proxy_authorization_t structure.
+ */
+
+/**@ingroup sip_proxy_authorization
+ * @typedef typedef struct sip_proxy_authorization_s sip_proxy_authorization_t;
+ *
+ * The structure #sip_proxy_authorization_t contains representation of SIP
+ * @ProxyAuthorization header.
+ *
+ * The #sip_proxy_authorization_t is defined as follows:
+ * @code
+ * typedef struct msg_auth_s {
+ *   msg_common_t       au_common[1];  // Common fragment info 
+ *   msg_auth_t        *au_next;       // Link to next header 
+ *   char const        *au_scheme;     // Auth-scheme like "Basic" or "Digest" 
+ *   msg_param_t const *au_params;     // Comma-separated parameters
+ * } sip_proxy_authorization_t;
+ * @endcode
+ *
  */
 
 msg_hclass_t sip_proxy_authorization_class[] =
 SIP_HEADER_CLASS_AUTH(proxy_authorization, "Proxy-Authorization", append);
 
-int sip_proxy_authorization_d(su_home_t *home,
-			      sip_header_t *h, char *s, int slen)
+issize_t sip_proxy_authorization_d(su_home_t *home, sip_header_t *h,
+				   char *s, isize_t slen)
 {
   return msg_auth_d(home, h, s, slen);
 }
 
-int sip_proxy_authorization_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_proxy_authorization_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   assert(sip_is_proxy_authorization(h));
   return msg_auth_e(b, bsiz, h, f);
@@ -218,17 +283,38 @@ int sip_proxy_authorization_e(char b[], int bsiz, sip_header_t const *h, int f)
  *
  * @todo Currently the parser fails if several challenges are included one
  * header.
+ *
+ * The parsed WWW-Authenticate header
+ * is stored in #sip_www_authenticate_t structure.
+ */
+
+/**@ingroup sip_www_authenticate
+ * @typedef typedef struct sip_www_authenticate_s sip_www_authenticate_t;
+ *
+ * The structure #sip_www_authenticate_t contains representation of SIP
+ * @WWWAuthenticate header.
+ *
+ * The #sip_www_authenticate_t is defined as follows:
+ * @code
+ * typedef struct msg_auth_s {
+ *   msg_common_t       au_common[1];  // Common fragment info 
+ *   msg_auth_t        *au_next;       // Link to next header 
+ *   char const        *au_scheme;     // Auth-scheme like "Basic" or "Digest" 
+ *   msg_param_t const *au_params;     // Comma-separated parameters
+ * } sip_www_authenticate_t;
+ * @endcode
+ *
  */
 
 msg_hclass_t sip_www_authenticate_class[] =
 SIP_HEADER_CLASS_AUTH(www_authenticate, "WWW-Authenticate", single);
 
-int sip_www_authenticate_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_www_authenticate_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
   return msg_auth_d(home, h, s, slen);
 }
 
-int sip_www_authenticate_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_www_authenticate_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   assert(sip_is_www_authenticate(h));
   return msg_auth_e(b, bsiz, h, f);
@@ -239,7 +325,7 @@ int sip_www_authenticate_e(char b[], int bsiz, sip_header_t const *h, int f)
  * The @b Authentication-Info header contains either a next-nonce used by
  * next request and/or authentication from server used in mutual
  * authentication. The syntax of @b Authentication-Info header is defined in
- * @RFC2617 and [S20.6] as follows:
+ * @RFC2617 and @RFC3261 as follows:
  *
  * @code
  *   Authentication-Info  = "Authentication-Info" HCOLON ainfo
@@ -251,6 +337,26 @@ int sip_www_authenticate_e(char b[], int bsiz, sip_header_t const *h, int f)
  *   response-auth        =  "rspauth" EQUAL response-digest
  *   response-digest      =  LDQUOT *LHEX RDQUOT
  * @endcode
+ *
+ * The parsed Authentication-Info header
+ * is stored in #sip_authentication_info_t structure.
+ */
+
+/**@ingroup sip_authentication_info
+ * @typedef typedef struct sip_authentication_info_s sip_authentication_info_t;
+ *
+ * The structure #sip_authentication_info_t contains representation of SIP
+ * @AuthenticationInfo header.
+ *
+ * The #sip_authentication_info_t is defined as follows:
+ * @code
+ * typedef struct msg_list_s
+ * {
+ *   msg_common_t       k_common[1];  // Common fragment info
+ *   msg_list_t        *k_next;	      // Dummy link to next header
+ *   msg_param_t       *k_items;      // List of ainfo
+ * } sip_authentication_info_t;
+ * @endcode
  */
 
 #define sip_authentication_info_dup_xtra msg_list_dup_xtra
@@ -261,13 +367,13 @@ msg_hclass_t sip_authentication_info_class[] =
   SIP_HEADER_CLASS(authentication_info, "Authentication-Info", "",
 		   ai_params, append, authentication_info);
 
-int sip_authentication_info_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_authentication_info_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
   return msg_list_d(home, (msg_header_t *)h, s, slen);
 }
 
 
-int sip_authentication_info_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_authentication_info_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   assert(sip_is_authentication_info(h));
   return msg_list_e(b, bsiz, h, f);
@@ -296,6 +402,27 @@ int sip_authentication_info_e(char b[], int bsiz, sip_header_t const *h, int f)
  *
  * @note @b Proxy-Authentication-Info is not specified @RFC3261 and it is
  * mentioned by @RFC2617 but in passage.
+ *
+ * The parsed Proxy-Authentication-Info header
+ * is stored in #sip_proxy_authentication_info_t structure.
+ */
+
+/**@ingroup sip_proxy_authentication_info
+ * @typedef typedef struct sip_proxy_authentication_info_s sip_proxy_authentication_info_t;
+ *
+ * The structure #sip_proxy_authentication_info_t contains representation of SIP
+ * @ProxyAuthenticationInfo header.
+ *
+ * The #sip_proxy_authentication_info_t is defined as follows:
+ * @code
+ * typedef struct msg_list_s
+ * {
+ *   msg_common_t       k_common[1];  // Common fragment info
+ *   msg_list_t        *k_next;	      // Dummy link to next header
+ *   msg_param_t       *k_items;      // List of ainfo
+ * } sip_proxy_authentication_info_t;
+ * @endcode
+ *
  */
 
 #define sip_proxy_authentication_info_dup_xtra msg_list_dup_xtra
@@ -306,18 +433,19 @@ msg_hclass_t sip_proxy_authentication_info_class[] =
   SIP_HEADER_CLASS(proxy_authentication_info, "Proxy-Authentication-Info", "",
 		   ai_params, append, proxy_authentication_info);
 
-int sip_proxy_authentication_info_d(su_home_t *home, 
-				    sip_header_t *h, char *s, int slen)
+issize_t sip_proxy_authentication_info_d(su_home_t *home, sip_header_t *h,
+					 char *s, isize_t slen)
 {
   return msg_list_d(home, (msg_header_t *)h, s, slen);
 }
 
-int sip_proxy_authentication_info_e(char b[], int bsiz, 
-				    sip_header_t const *h, int f)
+issize_t sip_proxy_authentication_info_e(char b[], isize_t bsiz, 
+					 sip_header_t const *h, int f)
 {
   assert(sip_is_authentication_info(h));
   return msg_list_e(b, bsiz, h, f);
 }
+
 /* ====================================================================== */
 
 /* Functions parsing @RFC3329 SIP Security Agreement headers */
@@ -326,54 +454,29 @@ typedef struct sip_security_agree_s sip_security_agree_t;
 #define sh_security_agree sh_security_client
 
 static 
-int sip_security_agree_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_security_agree_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
-  sip_header_t **hh = &h->sh_succ, *h0 = h;
-  sip_security_agree_t *sa = h->sh_security_agree;
+  sip_security_agree_t *sa = (sip_security_agree_t *)h;
 
-  int n;
+  isize_t n;
 
-  for (;*s;) {
-    /* Ignore empty entries (comma-whitespace) */
-    if (*s == ',') { 
-      *s++ = '\0'; skip_lws(&s); 
-      continue; 
-    }
+  while (*s == ',')   /* Ignore empty entries (comma-whitespace) */
+    *s = '\0', s += span_lws(s + 1) + 1;
 
-    if (!h) {      /* Allocate next header structure */
-      if (!(h = sip_header_alloc(home, h0->sh_class, 0)))
-	return -1;
-      *hh = h; h->sh_prev = hh; hh = &h->sh_succ;
-      sa = sa->sa_next = h->sh_security_agree;
-    }
+  if ((n = span_token(s)) == 0) 
+    return -1;
+  sa->sa_mec = s; s += n; while (IS_LWS(*s)) *s++ = '\0'; 
+  if (*s == ';' && msg_params_d(home, &s, &sa->sa_params) < 0)
+    return -1;
 
-    sa = h->sh_security_agree;
-
-    if ((n = span_token(s)) == 0) 
-      return -1;
-    sa->sa_mec = s; s += n; while (IS_LWS(*s)) *s++ = '\0'; 
-    if (*s == ';' && msg_params_d(home, &s, &sa->sa_params) < 0)
-      return -1;
-    if (*s != '\0' && *s != ',')
-      return -1;
-
-    if (sa->sa_params)
-      msg_header_update_params(sa->sa_common, 0);
-
-    h = NULL;
-  }
-
-  if (h)			/* Empty list -> error */
-     return -1;
-
-  return 0;
+  return msg_parse_next_field(home, h, s, slen);
 }
 
 static 
-int sip_security_agree_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_security_agree_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   char *end = b + bsiz, *b0 = b;
-  sip_security_agree_t const *sa = h->sh_security_agree;
+  sip_security_agree_t const *sa = (sip_security_agree_t const *)h;
 
   MSG_STRING_E(b, end, sa->sa_mec);
   MSG_PARAMS_E(b, end, sa->sa_params, flags);
@@ -382,7 +485,7 @@ int sip_security_agree_e(char b[], int bsiz, sip_header_t const *h, int f)
 }
 
 static 
-int sip_security_agree_dup_xtra(sip_header_t const *h, int offset)
+isize_t sip_security_agree_dup_xtra(sip_header_t const *h, isize_t offset)
 {
   sip_security_agree_t const *sa = h->sh_security_agree;
 
@@ -395,7 +498,7 @@ int sip_security_agree_dup_xtra(sip_header_t const *h, int offset)
 /** Duplicate one sip_security_agree_t object */ 
 static 
 char *sip_security_agree_dup_one(sip_header_t *dst, sip_header_t const *src,
-			char *b, int xtra)
+				 char *b, isize_t xtra)
 {
   sip_security_agree_t *sa_dst = dst->sh_security_agree;
   sip_security_agree_t const *sa_src = src->sh_security_agree;
@@ -403,13 +506,13 @@ char *sip_security_agree_dup_one(sip_header_t *dst, sip_header_t const *src,
   char *end = b + xtra;
   b = msg_params_dup(&sa_dst->sa_params, sa_src->sa_params, b, xtra);
   MSG_STRING_DUP(b, sa_dst->sa_mec, sa_src->sa_mec);
-  assert(b <= end);
+  assert(b <= end); (void)end;
 
   return b;
 }
 
 static int sip_security_agree_update(msg_common_t *h, 
-				     char const *name, int namelen,
+				     char const *name, isize_t namelen,
 				     char const *value)
 {
   sip_security_agree_t *sa = (sip_security_agree_t *)h;
@@ -467,19 +570,45 @@ static int sip_security_agree_update(msg_common_t *h,
  *    extension        = generic-param
  * @endcode
  *
- * @sa sip_security_server, sip_security_verify
+ * @sa @SecurityServer, @SecurityVerify, sip_security_verify_compare(),
+ * sip_security_client_select(), @RFC3329
+ *
+ * The parsed Security-Client header
+ * is stored in #sip_security_client_t structure.
+ */
+
+/**@ingroup sip_security_client
+ * @typedef typedef struct sip_security_client_s sip_security_client_t;
+ *
+ * The structure #sip_security_client_t contains representation of SIP
+ * @SecurityClient header.
+ *
+ * The #sip_security_client_t is defined as follows:
+ * @code
+ * typedef struct sip_security_agree_s
+ * {
+ *   sip_common_t        sa_common[1]; // Common fragment info 
+ *   sip_security_client_t *sa_next;   // Link to next mechanism 
+ *   char const         *sa_mec;       // Security mechanism
+ *   msg_param_t const  *sa_params;    // List of mechanism parameters
+ *   char const         *sa_q;         // Value of q (preference) parameter
+ *   char const         *sa_d_alg;     // Value of d-alg parameter
+ *   char const         *sa_d_qop;     // Value of d-qop parameter
+ *   char const         *sa_d_ver;     // Value of d-ver parameter
+ * } sip_security_client_t;
+ * @endcode
  */
 
 msg_hclass_t sip_security_client_class[] = 
 SIP_HEADER_CLASS(security_client, "Security-Client", "", 
 		 sa_params, append, security_agree);
 
-int sip_security_client_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_security_client_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
   return sip_security_agree_d(home, h, s, slen);
 }
 
-int sip_security_client_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_security_client_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   return sip_security_agree_e(b, bsiz, h, f);
 }
@@ -490,19 +619,45 @@ int sip_security_client_e(char b[], int bsiz, sip_header_t const *h, int f)
  * The Security-Server header is defined by @RFC3329, "Security Mechanism
  * Agreement for the Session Initiation Protocol (SIP)".
  *
- * @sa sip_security_client, sip_security_verify
+ * @sa @SecurityClient, @SecurityVerify, sip_security_verify_compare(),
+ * sip_security_client_select(), @RFC3329.
+ *
+ * The parsed Security-Server header
+ * is stored in #sip_security_server_t structure.
+ */
+
+/**@ingroup sip_security_server
+ * @typedef typedef struct sip_security_server_s sip_security_server_t;
+ *
+ * The structure #sip_security_server_t contains representation of SIP
+ * @SecurityServer header.
+ *
+ * The #sip_security_server_t is defined as follows:
+ * @code
+ * typedef struct sip_security_agree_s
+ * {
+ *   sip_common_t        sa_common[1]; // Common fragment info 
+ *   sip_security_server_t *sa_next;   // Link to next mechanism 
+ *   char const         *sa_mec;       // Security mechanism
+ *   msg_param_t const  *sa_params;    // List of mechanism parameters
+ *   char const         *sa_q;         // Value of q (preference) parameter
+ *   char const         *sa_d_alg;     // Value of d-alg parameter
+ *   char const         *sa_d_qop;     // Value of d-qop parameter
+ *   char const         *sa_d_ver;     // Value of d-ver parameter
+ * } sip_security_server_t;
+ * @endcode
  */
 
 msg_hclass_t sip_security_server_class[] = 
 SIP_HEADER_CLASS(security_server, "Security-Server", "", 
 		 sa_params, append, security_agree);
 
-int sip_security_server_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_security_server_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
   return sip_security_agree_d(home, h, s, slen);
 }
 
-int sip_security_server_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_security_server_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   return sip_security_agree_e(b, bsiz, h, f);
 }
@@ -513,19 +668,46 @@ int sip_security_server_e(char b[], int bsiz, sip_header_t const *h, int f)
  * The Security-Verify header is defined by @RFC3329, "Security Mechanism
  * Agreement for the Session Initiation Protocol (SIP)".
  *
- * @sa sip_security_client, sip_security_server
+ * @sa @SecurityClient, @SecurityServer, sip_security_verify_compare(),
+ * sip_security_client_select(), @RFC3329.
+ *
+ * The parsed Security-Verify header
+ * is stored in #sip_security_verify_t structure.
  */
 
+/**@ingroup sip_security_verify
+ * @typedef typedef struct sip_security_verify_s sip_security_verify_t;
+ *
+ * The structure #sip_security_verify_t contains representation of SIP
+ * @SecurityVerify header.
+ *
+ * The #sip_security_verify_t is defined as follows:
+ * @code
+ * typedef struct sip_security_agree_s
+ * {
+ *   sip_common_t        sa_common[1]; // Common fragment info 
+ *   sip_security_verify_t *sa_next;   // Link to next mechanism 
+ *   char const         *sa_mec;       // Security mechanism
+ *   msg_param_t const  *sa_params;    // List of mechanism parameters
+ *   char const         *sa_q;         // Value of q (preference) parameter
+ *   char const         *sa_d_alg;     // Value of d-alg parameter
+ *   char const         *sa_d_qop;     // Value of d-qop parameter
+ *   char const         *sa_d_ver;     // Value of d-ver parameter
+ * } sip_security_verify_t;
+ * @endcode
+ */
+
+
 msg_hclass_t sip_security_verify_class[] = 
 SIP_HEADER_CLASS(security_verify, "Security-Verify", "", 
 		 sa_params, append, security_agree);
 
-int sip_security_verify_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_security_verify_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
   return sip_security_agree_d(home, h, s, slen);
 }
 
-int sip_security_verify_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_security_verify_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   return sip_security_agree_e(b, bsiz, h, f);
 }
@@ -536,13 +718,31 @@ int sip_security_verify_e(char b[], int bsiz, sip_header_t const *h, int f)
 /**@SIP_HEADER sip_privacy Privacy Header
  * 
  * The Privacy header is used by User-Agent to request privacy services from
- * the network. Its syntax is defined in [RFC3323] as follows:
+ * the network. Its syntax is defined in @RFC3323 as follows:
  * 
  * @code
  *    Privacy-hdr  =  "Privacy" HCOLON priv-value *(";" priv-value)
  *    priv-value   =   "header" / "session" / "user" / "none" / "critical"
  *                     / token
  * @endcode
+ *
+ * The parsed Privacy header is stored in #sip_privacy_t structure.
+ */
+
+/**@ingroup sip_privacy
+ * @typedef typedef struct sip_privacy_s sip_privacy_t;
+ *
+ * The structure #sip_privacy_t contains representation of a SIP @Privacy
+ * header.
+ *
+ * The #sip_privacy_t is defined as follows:
+ * @code
+ * typedef struct sip_privacy_s {
+ *   sip_common_t       priv_common[1];	// Common fragment info 
+ *   sip_error_t       *priv_next;     	// Dummy link 
+ *   msg_param_t const *priv_values;   	// List of privacy values 
+ * } sip_privacy_t;
+ * @endcode
  */
 
 msg_xtra_f sip_privacy_dup_xtra;
@@ -554,7 +754,7 @@ msg_hclass_t sip_privacy_class[] =
 SIP_HEADER_CLASS(privacy, "Privacy", "", priv_values, single, privacy);
 
 static 
-int sip_privacy_token_scan(char *start)
+issize_t sip_privacy_token_scan(char *start)
 {
   char *s = start;
   skip_token(&s);
@@ -569,9 +769,9 @@ int sip_privacy_token_scan(char *start)
   return s - start;
 }
 
-int sip_privacy_d(su_home_t *home, sip_header_t *h, char *s, int slen)
+issize_t sip_privacy_d(su_home_t *home, sip_header_t *h, char *s, isize_t slen)
 {
-  sip_privacy_t *priv = h->sh_privacy;
+  sip_privacy_t *priv = (sip_privacy_t *)h;
 
   while (*s == ';' || *s == ',') {
     s++;
@@ -595,11 +795,11 @@ int sip_privacy_d(su_home_t *home, sip_header_t *h, char *s, int slen)
   }
 }
 
-int sip_privacy_e(char b[], int bsiz, sip_header_t const *h, int f)
+issize_t sip_privacy_e(char b[], isize_t bsiz, sip_header_t const *h, int f)
 {
   sip_privacy_t const *priv = h->sh_privacy;
   char *b0 = b, *end = b + bsiz;
-  int i;
+  size_t i;
 
   if (priv->priv_values) {
     for (i = 0; priv->priv_values[i]; i++) {
@@ -613,7 +813,7 @@ int sip_privacy_e(char b[], int bsiz, sip_header_t const *h, int f)
   return b - b0;
 }
 
-int sip_privacy_dup_xtra(sip_header_t const *h, int offset)
+isize_t sip_privacy_dup_xtra(sip_header_t const *h, isize_t offset)
 {
   sip_privacy_t const *priv = h->sh_privacy;
 
@@ -623,9 +823,9 @@ int sip_privacy_dup_xtra(sip_header_t const *h, int offset)
 }
 
 char *sip_privacy_dup_one(sip_header_t *dst,
-			      sip_header_t const *src,
-			      char *b,
-			      int xtra)
+			  sip_header_t const *src,
+			  char *b,
+			  isize_t xtra)
 {
   sip_privacy_t *priv = dst->sh_privacy;
   sip_privacy_t const *o = src->sh_privacy;
@@ -633,7 +833,7 @@ char *sip_privacy_dup_one(sip_header_t *dst,
 
   b = msg_params_dup(&priv->priv_values, o->priv_values, b, xtra);
 
-  assert(b <= end);
+  assert(b <= end); (void)end;
 
   return b;
 }
