@@ -71,7 +71,6 @@ typedef unsigned _int32 uint32_t;
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <pthread.h>
 #include <errno.h>
 
 #define TSTFLAGS tstflags
@@ -101,8 +100,6 @@ struct sres_context_s
   sres_query_t    *query;
   sres_record_t  **result;
 
-  pthread_mutex_t  mutex[1];
-
   int              timeout;
   int              sink;
   int              sinkidx;
@@ -110,7 +107,7 @@ struct sres_context_s
   
   int              ready;
   int              n_sockets;
-  int              sockets[SRES_MAX_NAMESERVERS];
+  sres_socket_t    sockets[SRES_MAX_NAMESERVERS];
 #if HAVE_POLL
   struct pollfd    fds[SRES_MAX_NAMESERVERS];
 #endif
@@ -124,8 +121,11 @@ static void test_answer_multi(sres_context_t *ctx, sres_query_t *query,
 static int tstflags = 0;
 
 #if 1
+
+#if HAVE_POLL && 0
+
 static
-int setblocking(int s, int blocking)
+int setblocking(sres_socket_t s, int blocking)
 {
   unsigned mode = fcntl(s, F_GETFL, 0);
 
@@ -140,21 +140,17 @@ int setblocking(int s, int blocking)
   return fcntl(s, F_SETFL, mode);
 }
 
-#if HAVE_POLL
-
 /** Test few assumptions about sockets */
 static
 int test_socket(sres_context_t *ctx)
 {
   int af;
-  int s1, s2, s3, s4;
+  su_sockeet_t s1, s2, s3, s4;
   struct sockaddr_storage a[1];
   struct sockaddr_storage a1[1], a2[1], a3[1], a4[1];
   struct sockaddr_in *sin = (void *)a;
-  struct sockaddr_in *sin1 = (void *)a1, *sin2 = (void *)a2;
   struct sockaddr_in *sin3 = (void *)a3, *sin4 = (void *)a4;
   struct sockaddr *sa = (void *)a;
-  struct sockaddr *sa1 = (void *)a1, *sa2 = (void *)a2;
   struct sockaddr *sa3 = (void *)a3, *sa4 = (void *)a4;
   socklen_t alen, a1len, a2len, a3len, a4len;
   char buf[16];
@@ -164,10 +160,10 @@ int test_socket(sres_context_t *ctx)
   af = AF_INET;
 
   for (;;) {
-    TEST_1((s1 = socket(af, SOCK_DGRAM, 0)) != -1);
-    TEST_1((s2 = socket(af, SOCK_DGRAM, 0)) != -1);
-    TEST_1((s3 = socket(af, SOCK_DGRAM, 0)) != -1);
-    TEST_1((s4 = socket(af, SOCK_DGRAM, 0)) != -1);
+    TEST_1((s1 = su_socket(af, SOCK_DGRAM, 0)) != INVALID_SOCKET);
+    TEST_1((s2 = su_socket(af, SOCK_DGRAM, 0)) != INVALID_SOCKET);
+    TEST_1((s3 = su_socket(af, SOCK_DGRAM, 0)) != INVALID_SOCKET);
+    TEST_1((s4 = su_socket(af, SOCK_DGRAM, 0)) != INVALID_SOCKET);
 
     TEST_1(setblocking(s1, 0) == 0);
     TEST_1(setblocking(s2, 0) == 0);
@@ -664,7 +660,7 @@ int test_a6(sres_context_t *ctx)
   TEST_S(inet_ntop(AF_INET6, &rr_a6->a6_suffix, buf, sizeof(buf)), 
 	 "3ffe:1200:3012:c000:210:a4ff:fe8d:6a46");
 
-  TEST(rr_a6->a6_prename, NULL);
+  TEST_P(rr_a6->a6_prename, NULL);
 
   sres_free_answers(res, ctx->result), ctx->result = NULL;
 
@@ -678,7 +674,7 @@ int test_a6(sres_context_t *ctx)
   TEST_S(inet_ntop(AF_INET6, &rr_a6->a6_suffix, buf, sizeof(buf)), 
 	 "3ffe:1200:3012:c000:210:a4ff:fe8d:6a46");
 
-  TEST(rr_a6->a6_prename, NULL);
+  TEST_P(rr_a6->a6_prename, NULL);
 
   sres_free_answers(res, result), result = NULL;
 
@@ -724,7 +720,7 @@ int test_a6_prefix(sres_context_t *ctx)
   TEST(rr_a6->a6_prelen, 0);
   TEST_S(inet_ntop(AF_INET6, &rr_a6->a6_suffix, buf, sizeof(buf)), 
 	 "3ff0:12:3012:c006:a08:20ff:fe7d:e7ac");
-  TEST(rr_a6->a6_prename, NULL);
+  TEST_P(rr_a6->a6_prename, NULL);
  
   sres_free_answers(res, ctx->result), ctx->result = NULL;
 
@@ -1108,7 +1104,7 @@ int test_cache(sres_context_t *ctx)
     rr = result[i];
 
     if (rr->sr_record->r_type == sres_type_naptr) {
-      sres_naptr_record_t *rr_naptr = rr->sr_naptr;
+      const sres_naptr_record_t *rr_naptr = rr->sr_naptr;
       switch(rr_naptr->na_order) {
       case 20:
         TEST(rr_naptr->na_record->r_type, sres_type_naptr);
@@ -1290,7 +1286,7 @@ int test_cache(sres_context_t *ctx)
 
   TEST_S(inet_ntop(AF_INET6, &rr_a6->a6_suffix, buf, sizeof(buf)), "3ff0::");
 
-  TEST(rr_a6->a6_prename, NULL);
+  TEST_P(rr_a6->a6_prename, NULL);
 
   sres_free_answers(res, result);
 
@@ -1346,7 +1342,7 @@ int test_query_one_type(sres_context_t *ctx)
   TEST_S(inet_ntop(AF_INET6, &rr_aaaa->aaaa_addr, buf, sizeof(buf)), 
 	 "3ffe:1200:3012:c000:206:5bff:fe55:462f");
 
-  TEST(result[1], NULL);
+  TEST_P(result[1], NULL);
 
   END();
 }
@@ -1452,7 +1448,7 @@ int test_timeout(sres_context_t *ctx)
   TEST_RUN(ctx);
   ctx->timeout = 0;
 
-  TEST(ctx->result, NULL);
+  TEST_P(ctx->result, NULL);
 
   result = sres_cached_answers(res, sres_type_a, domain);
 
@@ -1474,7 +1470,7 @@ int test_timeout(sres_context_t *ctx)
   END();
 }
 
-void test_answer(sres_context_t *ctx,
+static void test_answer(sres_context_t *ctx,
 		 sres_query_t *q,
 		 sres_record_t **answer)
 {
@@ -1485,7 +1481,7 @@ void test_answer(sres_context_t *ctx,
   BREAK(ctx);
 }
 
-void test_answer_multi(sres_context_t *ctx,
+static void test_answer_multi(sres_context_t *ctx,
 		       sres_query_t *q,
 		       sres_record_t **answer)
 {
@@ -1534,7 +1530,7 @@ int test_expiration(sres_context_t *ctx)
 
   result = sres_cached_answers(res, sres_qtype_any, domain);
 
-  TEST(result, NULL);  /* the cache should be empty after 15 secs */
+  TEST_P(result, NULL);  /* the cache should be empty after 15 secs */
 
   END();
 }
@@ -1740,39 +1736,39 @@ int test_api_errors(sres_context_t *noctx)
   
   s = sockets[0];
   
-  TEST(sres_resolver_ref(NULL), NULL);
+  TEST_P(sres_resolver_ref(NULL), NULL);
   TEST(errno, EFAULT);
   sres_resolver_unref(NULL);
 
-  TEST(sres_resolver_set_userdata(NULL, NULL), NULL);
+  TEST_P(sres_resolver_set_userdata(NULL, NULL), NULL);
   TEST(errno, EFAULT);
 
-  TEST(sres_resolver_get_userdata(NULL), NULL);
+  TEST_P(sres_resolver_get_userdata(NULL), NULL);
 
-  TEST(sres_resolver_get_userdata(res), NULL);
-  TEST(sres_resolver_set_userdata(res, sa), NULL);
-  TEST(sres_resolver_get_userdata(res), sa);
-  TEST(sres_resolver_set_userdata(res, NULL), sa); 
-  TEST(sres_resolver_get_userdata(res), NULL);
+  TEST_P(sres_resolver_get_userdata(res), NULL);
+  TEST_P(sres_resolver_set_userdata(res, sa), NULL);
+  TEST_P(sres_resolver_get_userdata(res), sa);
+  TEST_P(sres_resolver_set_userdata(res, NULL), sa); 
+  TEST_P(sres_resolver_get_userdata(res), NULL);
 
   errno = 0;
-  TEST(sres_query(NULL, test_answer, ctx, sres_type_a, "com"), NULL);
+  TEST_P(sres_query(NULL, test_answer, ctx, sres_type_a, "com"), NULL);
   TEST(errno, EFAULT); errno = 0;
-  TEST(sres_query(res, test_answer, ctx, sres_type_a, NULL), NULL);
+  TEST_P(sres_query(res, test_answer, ctx, sres_type_a, NULL), NULL);
   TEST(errno, EFAULT); errno = 0;
-  TEST(sres_query_sockaddr(res, test_answer, ctx,
-				sres_qtype_any, sa), NULL);
+  TEST_P(sres_query_sockaddr(res, test_answer, ctx,
+			     sres_qtype_any, sa), NULL);
 #if defined(EAFNOSUPPORT)
   TEST(errno, EAFNOSUPPORT); errno = 0;
 #endif
 
-  TEST(sres_cached_answers(NULL, sres_qtype_any, "example.com"), NULL);
+  TEST_P(sres_cached_answers(NULL, sres_qtype_any, "example.com"), NULL);
   TEST(errno, EFAULT); errno = 0;
-  TEST(sres_cached_answers(res, sres_qtype_any, NULL), NULL);
+  TEST_P(sres_cached_answers(res, sres_qtype_any, NULL), NULL);
   TEST(errno, EFAULT); errno = 0;
-  TEST(sres_cached_answers(res, sres_qtype_any, name2048), NULL);
+  TEST_P(sres_cached_answers(res, sres_qtype_any, name2048), NULL);
   TEST(errno, ENAMETOOLONG); errno = 0;
-  TEST(sres_cached_answers_sockaddr(res, sres_qtype_any, sa), NULL);
+  TEST_P(sres_cached_answers_sockaddr(res, sres_qtype_any, sa), NULL);
   TEST(errno, EAFNOSUPPORT); errno = 0;
 
   sres_free_answer(res, NULL);
@@ -1811,7 +1807,7 @@ int test_init(sres_context_t *ctx, char const *conf_file)
     ctx->fds[i].events = POLLIN | POLLERR;
   }
   
-  TEST(sres_resolver_ref(ctx->resolver), ctx->resolver);
+  TEST_P(sres_resolver_ref(ctx->resolver), ctx->resolver);
   sres_resolver_unref(ctx->resolver);
 
   END();
@@ -1841,7 +1837,7 @@ static
 int test_conf_errors(sres_context_t *ctx, char const *conf_file)
 {
   sres_resolver_t *res;
-  int socket;
+  sres_socket_t socket;
   int n;
 
   BEGIN();
@@ -1865,7 +1861,7 @@ port $port
   printf("%s:%u: %s test should be updated\n", 
 	 __FILE__, __LINE__, __func__);
 #else
-  TEST(sres_query(res, test_answer, ctx, sres_type_a, "example.com"), NULL);
+  TEST_P(sres_query(res, test_answer, ctx, sres_type_a, "example.com"), NULL);
 #endif
     
   sres_resolver_unref(res);
