@@ -52,6 +52,11 @@
 #include <sofia-sip/su.h>
 #include <sofia-sip/su_localinfo.h>
 
+#if HAVE_WINSOCK2_H
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
+
 #if defined(HAVE_OPENSSL)
 #include <openssl/opensslv.h>
 #endif
@@ -73,7 +78,18 @@ static char const __func__[] = "stun";
 #endif
 
 /** STUN log. */
-su_log_t stun_log[] = { SU_LOG_INIT("stun", "STUN_DEBUG", SU_DEBUG) }; 
+su_log_t stun_log[] = { SU_LOG_INIT("stun", "STUN_DEBUG", 3) }; 
+
+/**@var STUN_DEBUG
+ *
+ * Environment variable determining the debug log level for @b stun module.
+ *
+ * The STUN_DEBUG environment variable is used to determine the debug logging
+ * level for @b stun module. The default level is 3.
+ * 
+ * @sa <sofia-sip/su_debug.h>, stun_log, SOFIA_DEBUG
+ */
+extern char const STUN__DEBUG[];
 
 enum {
   STUN_SENDTO_TIMEOUT = 1000,
@@ -325,6 +341,10 @@ static int stun_send_binding_request(stun_request_t *req,
 			      su_sockaddr_t *srvr_addr);
 static int stun_bind_callback(stun_magic_t *m, su_wait_t *w, su_wakeup_arg_t *arg);
 
+#if defined (__CYGWIN__)
+static int get_localinfo(int family, su_sockaddr_t *su, socklen_t *return_len);
+#endif
+
 /* timers */
 static void stun_sendto_timer_cb(su_root_magic_t *magic, 
 				 su_timer_t *t,
@@ -510,7 +530,7 @@ int stun_obtain_shared_secret(stun_handle_t *sh,
   int events = -1;
   int one, err = -1;
   su_wait_t wait[1] = { SU_WAIT_INIT };
-  su_socket_t s = SOCKET_ERROR;
+  su_socket_t s = INVALID_SOCKET;
   int family;
   su_addrinfo_t *ai = NULL;
   stun_discovery_t *sd;
@@ -547,21 +567,18 @@ int stun_obtain_shared_secret(stun_handle_t *sh,
   /* open tcp connection to server */
   s = su_socket(family = AF_INET, SOCK_STREAM, 0);
 
-  if (s == -1) {
-    STUN_ERROR(errno, socket);
-    return -1;
+  if (s == INVALID_SOCKET) {
+    return STUN_ERROR(errno, socket);
   }
 
   /* asynchronous connect() */
   if (su_setblocking(s, 0) < 0) {
-    STUN_ERROR(errno, su_setblocking);
-    return -1;
+    return STUN_ERROR(errno, su_setblocking);
   }
 
   if (setsockopt(s, SOL_TCP, TCP_NODELAY,
 		 (void *)&one, sizeof one) == -1) {
-    STUN_ERROR(errno, setsockopt);
-    return -1;
+    return STUN_ERROR(errno, setsockopt);
   }
 
   /* Do an asynchronous connect(). Three error codes are ok,
@@ -570,8 +587,7 @@ int stun_obtain_shared_secret(stun_handle_t *sh,
 	      ai->ai_addrlen) == SOCKET_ERROR) {
     err = su_errno();
     if (err != EINPROGRESS && err != EAGAIN && err != EWOULDBLOCK) {
-      STUN_ERROR(err, connect);
-      return -1;
+      return STUN_ERROR(err, connect);
     }
   }
 
@@ -584,14 +600,13 @@ int stun_obtain_shared_secret(stun_handle_t *sh,
 
   events = SU_WAIT_CONNECT | SU_WAIT_ERR;
   if (su_wait_create(wait, s, events) == -1)
-    STUN_ERROR(errno, su_wait_create);
+    return STUN_ERROR(errno, su_wait_create);
 
   /* su_root_eventmask(sh->sh_root, sh->sh_root_index, s, events); */
 
   if ((sd->sd_index =
        su_root_register(sh->sh_root, wait, stun_tls_callback, (su_wakeup_arg_t *) sd, 0)) == -1) {
-    STUN_ERROR(errno, su_root_register);
-    return -1;
+    return STUN_ERROR(errno, su_root_register);
   }
 
   ta_start(ta, tag, value);
@@ -646,10 +661,7 @@ static stun_request_t *stun_request_create(stun_discovery_t *sd)
   memcpy(req->sr_local_addr, sd->sd_bind_addr, sizeof(su_sockaddr_t));
 
   /* Insert this request to the request queue */
-  if (sh->sh_requests)
-    x_insert(sh->sh_requests, req, sr);
-  else
-    sh->sh_requests = req;
+  x_insert(sh->sh_requests, req, sr);
 
   return req;
 }
@@ -666,9 +678,6 @@ void stun_request_destroy(stun_request_t *req)
   if (x_is_inserted(req, sr))
     x_remove(req, sr);
 
-  if (!x_is_inserted(req, sr) && !req->sr_next)
-    sh->sh_requests = NULL;
-
   req->sr_handle = NULL;
   req->sr_discovery = NULL;
   /* memset(req->sr_destination, 0, sizeof(su_sockaddr_t)); */
@@ -738,24 +747,24 @@ void stun_handle_destroy(stun_handle_t *sh)
 
 
 /** Create wait object and register it to the handle callback */
-int assign_socket(stun_discovery_t *sd, su_socket_t s, int reg_socket) 
+static
+int assign_socket(stun_discovery_t *sd, su_socket_t s, int register_socket) 
 {
   stun_handle_t *sh = sd->sd_handle;
   int events;
   stun_discovery_t *tmp;
   /* su_localinfo_t clientinfo[1]; */
-  su_sockaddr_t bind_addr;
-  socklen_t bind_len;
-  char ipaddr[SU_ADDRSIZE + 2] = { 0 };
-  su_sockaddr_t *sa;
+  su_sockaddr_t su[1];
+  socklen_t sulen;
+  char addr[SU_ADDRSIZE];
   int err;
   
   su_wait_t wait[1] = { SU_WAIT_INIT };
 
   enter;
 
-  if (s == -1) {
-    SU_DEBUG_3(("%s: invalid socket.\n", __func__));
+  if (s == INVALID_SOCKET) {
+    SU_DEBUG_3(("%s: invalid socket\n", __func__));
     return errno = EINVAL, -1;
   }
 
@@ -769,79 +778,62 @@ int assign_socket(stun_discovery_t *sd, su_socket_t s, int reg_socket)
   }
   sd->sd_socket = s;
 
-  if (reg_socket != 1)
+  if (!register_socket)
     return 0;
 
   /* set socket asynchronous */
   if (su_setblocking(s, 0) < 0) {
-    STUN_ERROR(errno, su_setblocking);
-
-    su_close(s);
-    return -1;
+    return STUN_ERROR(errno, su_setblocking);
   }
 
   /* xxx -- check if socket is already assigned to this root */
+
+  memset(su, 0, sulen = sizeof su);
+
+  /* Try to bind it if not already bound */
+  if (getsockname(s, &su->su_sa, &sulen) == -1 || su->su_port == 0) {
+
+    sulen = su->su_len = sizeof su->su_sin;
+    su->su_family = AF_INET;
+
+#if defined (__CYGWIN__)
+    get_localinfo(AF_INET, su, &sulen);
+#endif
+
+    if ((err = bind(s, &su->su_sa, sulen)) < 0) {
+      SU_DEBUG_3(("%s: bind(%s:%u): %s\n",  __func__, 
+		  inet_ntop(su->su_family, SU_ADDR(su), addr, sizeof(addr)),
+		  (unsigned) ntohs(su->su_port),
+		  su_strerror(su_errno())));
+      return -1;
+    }
+
+    if (getsockname(s, &su->su_sa, &sulen) == -1) {
+      return STUN_ERROR(errno, getsockname);
+    }
+  }
+
+  memcpy(&sd->sd_bind_addr, su, sulen);
+
+  SU_DEBUG_3(("%s: local socket is bound to %s:%u\n", __func__,
+	      inet_ntop(su->su_family, SU_ADDR(su), addr, sizeof(addr)),
+	      (unsigned) ntohs(su->su_port)));
+
   events = SU_WAIT_IN | SU_WAIT_ERR;
 
   if (su_wait_create(wait, s, events) == -1) {
-    STUN_ERROR(su_errno(), su_wait_create);
-    return -1;
+    return STUN_ERROR(su_errno(), su_wait_create);
   }
 
   /* Register receiving function with events specified above */
   if ((sd->sd_index = su_root_register(sh->sh_root,
 				       wait, stun_bind_callback,
 				       (su_wakeup_arg_t *) sd, 0)) < 0) {
-    STUN_ERROR(errno, su_root_register);
-    return -1;
+    return STUN_ERROR(errno, su_root_register);
   }
 
   SU_DEBUG_7(("%s: socket registered.\n", __func__));
 
-  sa = (void *) &bind_addr;
-  bind_len = SU_SOCKADDR_SIZE(sa);
-  memset(sa, 0, bind_len);
-  /* if bound check the error */
-  err = getsockname(s, (struct sockaddr *) sa, &bind_len);
-  if (err < 0 && errno == SOCKET_ERROR) {
-    STUN_ERROR(errno, getsockname);
-    return -1;
-  }
-
-  /* Not bound - bind it */
-  if (sa->su_port == 0) {
-#if defined (__CYGWIN__)
-    get_localinfo(clientinfo);
-#endif
-
-    sa->su_family = AF_INET;
-
-    if ((err = bind(s, (struct sockaddr *) sa, bind_len)) < 0) {
-      STUN_ERROR(errno, bind);
-      SU_DEBUG_3(("%s: Error binding to %s:%u\n", __func__, 
-		  inet_ntop(sa->su_family, SU_ADDR(sa), 
-			    ipaddr, sizeof(ipaddr)),
-		  (unsigned) ntohs(sa->su_port)));
-      return -1;
-    }
-
-    /* bind_len = clientinfo->li_addrlen; */
-    /* clientinfo->li_addrlen = bind_len; */
-    sa->su_len = bind_len; /* ? */
-  }
-
-  memcpy(&sd->sd_bind_addr, &bind_addr, sizeof bind_addr);
-
-  if (getsockname(s, (struct sockaddr *) &bind_addr, &bind_len) != 0) {
-    STUN_ERROR(errno, getsockname);
-    return -1;
-  }
-
-  SU_DEBUG_3(("%s: local socket is bound to %s:%u\n", __func__,
-	      inet_ntop(bind_addr.su_family, SU_ADDR(&bind_addr), 
-			ipaddr, sizeof(ipaddr)),
-	      (unsigned) ntohs(bind_addr.su_port)));
-
   return 0;
 }
 
@@ -850,48 +842,42 @@ int assign_socket(stun_discovery_t *sd, su_socket_t s, int reg_socket)
  * Helper function needed by Cygwin builds.
  */
 #if defined (__CYGWIN__)
-static int get_localinfo(su_localinfo_t *clientinfo)
+static int get_localinfo(int family, su_sockaddr_t *su, socklen_t *return_len)
 {
-  su_localinfo_t  hints[1] = {{ LI_CANONNAME | LI_NUMERIC }}, *li, *res = NULL;
-  su_sockaddr_t *sa;
-  int i, error, found = 0;
-  char ipaddr[SU_ADDRSIZE + 2] = { 0 };
+  su_localinfo_t hints[1] = {{ LI_CANONNAME | LI_NUMERIC }}, *li, *res = NULL;
+  int i, error;
+  char addr[SU_ADDRSIZE];
 
+  hints->li_family = family;
 
-  hints->li_family = AF_INET;
   if ((error = su_getlocalinfo(hints, &res)) == 0) {
-    
     /* try to bind to the first available address */
     for (i = 0, li = res; li; li = li->li_next) {
-      if (li->li_family != AF_INET)
+      if (li->li_family != family)
 	continue;
-      
-      clientinfo->li_family = li->li_family;
-      clientinfo->li_addrlen = li->li_addrlen;
-      
-      sa = clientinfo->li_addr;
-      memcpy(sa, li->li_addr, sizeof(su_sockaddr_t));
-      SU_DEBUG_3(("%s: local address found to be %s.\n",
-		  __func__, 
-		  inet_ntop(clientinfo->li_family, SU_ADDR(sa),
-			    ipaddr, sizeof(ipaddr))));
-      found = 1;
+      if (li->li_scope == LI_SCOPE_HOST)
+	continue;
+
+      assert(*return_len >= li->li_addrlen);
+
+      memcpy(su, li->li_addr, *return_len = li->li_addrlen);
+
+      SU_DEBUG_3(("%s: using local address %s\n", __func__, 
+		  inet_ntop(family, SU_ADDR(su), addr, sizeof(addr))));
       break;
     }
+
+    su_freelocalinfo(res);
     
-    if (!found) {
-      STUN_ERROR(error, su_getlocalinfo);
-      return -1;
+    if (!li) {			/* Not found */
+      return STUN_ERROR(error, su_getlocalinfo);
     }
+
+    return 0;
   }
   else {
-    STUN_ERROR(error, su_getlocalinfo);
-    return -1;
+    return STUN_ERROR(error, su_getlocalinfo);
   }
-  if (res)
-    su_freelocalinfo(res);
-
-  return 0;
 }
 #endif
 
@@ -1011,11 +997,12 @@ static int priv_stun_bind_send(stun_handle_t *sh, stun_request_t *req, stun_disc
  * @param sh       pointer to valid stun handle
  * @param sdf      callback to signal process progress
  * @param magic    context pointer attached to 'sdf'
+ * @param tag, value, ... list of tagged parameters.
  *
  * @TAGS
- * @TAG STUNTAG_SOCKET Bind socket for STUN (socket handle).
- * @TAG STUNTAG_REGISTER_SOCKET Register socket for eventloop owned by STUN (boolean)
-
+ * @TAG STUNTAG_SOCKET() Bind socket handle to STUN (socket handle).
+ * @TAG STUNTAG_REGISTER_SOCKET() Register socket for eventloop owned by STUN (boolean)
+ *
  * @return
  * On success, zero is returned.  Upon error, -1 is returned, and @e errno is
  * set appropriately.
@@ -1037,11 +1024,11 @@ int stun_bind(stun_handle_t *sh,
 	      tag_type_t tag, tag_value_t value,
 	      ...)
 {
-  su_socket_t s = -1;
+  su_socket_t s = INVALID_SOCKET;
   stun_request_t *req = NULL;
   stun_discovery_t *sd = NULL;
   ta_list ta;
-  int index, s_reg = 0;
+  int s_reg = 0;
   
   enter;
 
@@ -1069,7 +1056,7 @@ int stun_bind(stun_handle_t *sh,
   ta_end(ta);
 
   sd = stun_discovery_create(sh, stun_action_binding_request, sdf, magic);
-  if ((index = assign_socket(sd, s, s_reg)) < 0)
+  if (assign_socket(sd, s, s_reg) < 0)
     return -1;
 
   req = stun_request_create(sd);
@@ -1098,7 +1085,7 @@ int stun_discovery_get_address(stun_discovery_t *sd,
 			       void *addr,
 			       socklen_t *return_addrlen)
 {
-  int siz;
+  socklen_t siz;
   
   enter;
 
@@ -1137,14 +1124,11 @@ static stun_discovery_t *stun_discovery_create(stun_handle_t *sh,
   sd->sd_lt = STUN_LIFETIME_EST;
   sd->sd_lt_max = STUN_LIFETIME_MAX;
 
-  sd->sd_pri_info.ai_addrlen = 16;
+  sd->sd_pri_info.ai_addrlen = sizeof sd->sd_pri_addr->su_sin;
   sd->sd_pri_info.ai_addr = &sd->sd_pri_addr->su_sa;
 
   /* Insert this action to the discovery queue */
-  if (sh->sh_discoveries)
-    x_insert(sh->sh_discoveries, sd, sd);
-  else
-    sh->sh_discoveries = sd;
+  x_insert(sh->sh_discoveries, sd, sd);
 
   return sd;
 }
@@ -1166,9 +1150,6 @@ static int stun_discovery_destroy(stun_discovery_t *sd)
   /* if we are in the queue*/
   if (x_is_inserted(sd, sd))
     x_remove(sd, sd);
-  /* if we were the only one */
-  else if (!sd->sd_next)
-    sh->sh_discoveries = NULL;
 
   sd->sd_next = NULL;
 
@@ -1205,9 +1186,7 @@ int stun_test_nattype(stun_handle_t *sh,
   char const *server = NULL;
   stun_request_t *req = NULL;
   stun_discovery_t *sd = NULL;
-  su_sockaddr_t bind_addr;
-  su_socket_t s = -1;
-  socklen_t bind_len;
+  su_socket_t s = INVALID_SOCKET;
   su_sockaddr_t *destination = NULL;
 
   enter;
@@ -1233,8 +1212,6 @@ int stun_test_nattype(stun_handle_t *sh,
 
   ta_end(ta);
 
-  bind_len = sizeof bind_addr;
-
   if (s < 0)
     return errno = EFAULT, -1;
 
@@ -1327,11 +1304,10 @@ static int stun_tls_callback(su_root_magic_t *m, su_wait_t *w, su_wakeup_arg_t *
 	      events & SU_WAIT_IN      ? " IN"        : "",
 	      events & SU_WAIT_OUT     ? " OUT"       : ""));
 
-
   getsockopt(sd->sd_socket, SOL_SOCKET, SO_ERROR,
 	     (void *)&one, &onelen);
   if (one != 0) {
-    STUN_ERROR(one, getsockopt);
+    STUN_ERROR(one, SO_ERROR);
   }
 
   if (one || events & SU_WAIT_ERR) {
@@ -1496,7 +1472,7 @@ static int stun_tls_callback(su_root_magic_t *m, su_wait_t *w, su_wakeup_arg_t *
     /* closed TLS connection */
     SSL_shutdown(ssl);
 
-    su_close(sd->sd_socket);
+    su_close(sd->sd_socket), sd->sd_socket = INVALID_SOCKET;
 
     SSL_free(self->sh_ssl), ssl = NULL;
     SSL_CTX_free(self->sh_ctx), ctx = NULL;
@@ -1678,7 +1654,7 @@ static int stun_bind_callback(stun_magic_t *m, su_wait_t *w, su_wakeup_arg_t *ar
   stun_handle_t *self = sd->sd_handle;
 
   int retval = -1, err = -1, dgram_len;
-  char ipaddr[SU_ADDRSIZE + 2];
+  char addr[SU_ADDRSIZE];
   stun_msg_t binding_response, *msg;
   unsigned char dgram[512] = { 0 };
   su_sockaddr_t recv;
@@ -1702,8 +1678,7 @@ static int stun_bind_callback(stun_magic_t *m, su_wait_t *w, su_wakeup_arg_t *ar
 
   /* receive response */
   recv_len = sizeof(recv);
-  dgram_len = recvfrom(s, dgram, sizeof(dgram), 0, (struct sockaddr *) &recv,
-		 &recv_len);
+  dgram_len = su_recvfrom(s, dgram, sizeof(dgram), 0, &recv, &recv_len);
   err = errno;
   if ((dgram_len < 0) && (err != EAGAIN)) {
     /* su_wait_destroy(w); */
@@ -1725,7 +1700,7 @@ static int stun_bind_callback(stun_magic_t *m, su_wait_t *w, su_wakeup_arg_t *ar
 
   
   SU_DEBUG_5(("%s: response from server %s:%u\n", __func__,
-	      inet_ntop(recv.su_family, SU_ADDR(&recv), ipaddr, sizeof(ipaddr)),
+	      inet_ntop(recv.su_family, SU_ADDR(&recv), addr, sizeof(addr)),
 	      ntohs(recv.su_port)));
 
   debug_print(&binding_response.enc_buf);      
@@ -2037,46 +2012,39 @@ static int action_bind(stun_request_t *req, stun_msg_t *binding_response)
 
 /**
  * Returns a non-zero value if some local interface address
- * matches 'sockaddr'.
+ * matches address in su.
  */
-static int priv_find_matching_localadress(su_sockaddr_t *sockaddr)
+static int priv_find_matching_localadress(su_sockaddr_t *su)
 {
-  su_localinfo_t  hints[1] = {{ LI_CANONNAME | LI_NUMERIC }}, *li, *res = NULL;
-  int i, error, found = 0;
-  char ipaddr[SU_ADDRSIZE + 2] = { 0 };
+  su_localinfo_t hints[1] = {{ LI_CANONNAME | LI_NUMERIC }}, *li, *res = NULL;
+  int af;
+  char addr[SU_ADDRSIZE];
 
-  SU_DEBUG_5(("%s: checking if %s is a local address.\n",
-	      __func__, 
-	      inet_ntop(AF_INET, SU_ADDR(sockaddr),
-			ipaddr, sizeof(ipaddr))));
+  SU_DEBUG_5(("%s: checking if %s is a local address.\n", __func__, 
+	      inet_ntop(AF_INET, SU_ADDR(su), addr, sizeof(addr))));
 
-  hints->li_family = AF_INET;
-  if ((error = su_getlocalinfo(hints, &res)) == 0) {
-    
-    /* check if any of the address match 'sockaddr'  */
-    for (i = 0, li = res; li; li = li->li_next) {
-      if (li->li_family != AF_INET)
-	continue;
+  hints->li_family = af = su->su_family;
+
+  if (su_getlocalinfo(hints, &res) != 0)
+    return 0;
+
+  /* check if any of the address match 'sockaddr'  */
+  for (li = res; li; li = li->li_next) {
+    if (li->li_family != af)
+      continue;
       
-      if (memcmp(SU_ADDR(sockaddr), SU_ADDR(li->li_addr), SU_ADDRLEN(sockaddr)) == 0
-) {
-	SU_DEBUG_5(("%s: found matching local address %s.\n",
-		    __func__, 
-		    inet_ntop(li->li_family, SU_ADDR(li->li_addr),
-			      ipaddr, sizeof(ipaddr))));
-	found = 1;
-	break;
-      }
-      else {
-	SU_DEBUG_9(("%s: skipping local address %s.\n",
-		    __func__, 
-		    inet_ntop(li->li_family, SU_ADDR(li->li_addr),
-			      ipaddr, sizeof(ipaddr))));
-      }
+    if (memcmp(SU_ADDR(su), SU_ADDR(li->li_addr), SU_ADDRLEN(su)) == 0) {
+      SU_DEBUG_5(("%s: found matching local address\n", __func__));
+      break;
     }
+
+    SU_DEBUG_9(("%s: skipping local address %s.\n", __func__, 
+		inet_ntop(af, SU_ADDR(li->li_addr), addr, sizeof(addr))));
   }  
 
-  return found;
+  su_freelocalinfo(res);
+
+  return li != NULL;
 }
 
 /**
@@ -2411,7 +2379,7 @@ static int stun_send_binding_request(stun_request_t *req,
 				     su_sockaddr_t  *srvr_addr)
 {
   su_timer_t *sendto_timer = NULL;
-  int s;
+  su_socket_t s;
   stun_handle_t *sh = req->sr_handle;
   stun_msg_t *msg =  req->sr_msg;
 
@@ -2597,19 +2565,27 @@ int stun_process_error_response(stun_msg_t *msg)
  */
 int stun_set_uname_pwd(stun_handle_t *sh,
 		       const char *uname,
-		       int len_uname,
+		       isize_t len_uname,
 		       const char *pwd,
-		       int len_pwd)
+		       isize_t len_pwd)
 {
   enter;
 
-  sh->sh_username.data = (unsigned char *) malloc(len_uname);
-  memcpy(sh->sh_username.data, uname, len_uname);
-  sh->sh_username.size = len_uname;
+  sh->sh_username.data = malloc(len_uname);
+  if (sh->sh_username.data) {
+    memcpy(sh->sh_username.data, uname, len_uname);
+    sh->sh_username.size = (unsigned)len_uname;
+  }
+  else
+    return -1;
   
-  sh->sh_passwd.data = (unsigned char *) malloc(len_pwd);
-  memcpy(sh->sh_passwd.data, pwd, len_pwd);
-  sh->sh_passwd.size = len_pwd;
+  sh->sh_passwd.data = malloc(len_pwd);
+  if (sh->sh_passwd.data) {
+    memcpy(sh->sh_passwd.data, pwd, len_pwd);
+    sh->sh_passwd.size = (unsigned)len_pwd;
+  }
+  else
+    return -1;
 
   sh->sh_use_msgint = 1; /* turn on message integrity ussage */
   
@@ -2645,7 +2621,7 @@ int stun_atoaddr(su_home_t *home,
     host = in;
   }
   else {
-    assert(port - in < strlen(in) + 1);
+    assert((size_t)(port - in) < strlen(in) + 1);
     memcpy(tmp, in, port - in);
     tmp[port - in] = 0;
     host = tmp;
@@ -2705,8 +2681,9 @@ int stun_test_lifetime(stun_handle_t *sh,
   stun_request_t *req = NULL;
   stun_discovery_t *sd = NULL;
   ta_list ta;
-  int s = -1, err, index = 0, s_reg = 0;
-  char ipaddr[SU_ADDRSIZE + 2] = { 0 };
+  su_socket_t s = INVALID_SOCKET;
+  int err, index = 0, s_reg = 0;
+  char addr[SU_ADDRSIZE];
   char const *server = NULL;
   su_socket_t sockfdy;
   socklen_t y_len;
@@ -2757,10 +2734,10 @@ int stun_test_lifetime(stun_handle_t *sh,
   /* ci = &req->sr_localinfo; */
 
   /* get local ip address */
-  /* get_localinfo(ci); */
+  /* get_localinfo(); */
 
   /* initialize socket y */
-  sockfdy = socket(AF_INET, SOCK_DGRAM, 0);
+  sockfdy = su_socket(AF_INET, SOCK_DGRAM, 0);
 
   /* set socket asynchronous */
   if (su_setblocking(sockfdy, 0) < 0) {
@@ -2787,7 +2764,7 @@ int stun_test_lifetime(stun_handle_t *sh,
   }
 
   SU_DEBUG_3(("%s: socket y bound to %s:%u\n", __func__,
-	      inet_ntop(y_addr.su_family, SU_ADDR(&y_addr), ipaddr, sizeof(ipaddr)),
+	      inet_ntop(y_addr.su_family, SU_ADDR(&y_addr), addr, sizeof(addr)),
 	      (unsigned) ntohs(y_addr.su_port)));
 
   req->sr_from_y = -1;
@@ -2852,25 +2829,25 @@ int stun_msg_is_keepalive(uint16_t data)
 }
 
 /**
- * Determines length of STUN message (0 if not stun). 
+ * Determines length of STUN message (0 (-1?) if not stun). 
  */
-int stun_message_length(void *data, int len, int end_of_message)
+int stun_message_length(void *data, isize_t len, int end_of_message)
 {
   unsigned char *p;
-  uint16_t tmp16, msg_type;
+  uint16_t msg_type;
+
+  if (len < 4)
+    return -1;
+
   /* parse header first */
   p = data;
-  memcpy(&tmp16, p, 2);
-  msg_type = ntohs(tmp16);
+  msg_type = (p[0] << 8) | p[1];
 
   if (msg_type == BINDING_REQUEST ||
       msg_type == BINDING_RESPONSE ||
       msg_type == BINDING_ERROR_RESPONSE) {
-    p+=2;
-    memcpy(&tmp16, p, 2);
-
     /* return message length */
-    return ntohs(tmp16);
+    return (p[0] << 8) | p[1];
   }
   else
     return -1;
@@ -2879,16 +2856,19 @@ int stun_message_length(void *data, int len, int end_of_message)
 /** Process incoming message */
 int stun_process_message(stun_handle_t *sh, su_socket_t s, 
 			 su_sockaddr_t *sa, socklen_t salen,
-			 void *data, int len)
+			 void *data, isize_t len)
 {
   int retval = -1;
   stun_msg_t msg;
 
   enter;
 
+  if (len >= 65536)
+    len = 65536;
+
   /* Message received. */
   msg.enc_buf.data = data;
-  msg.enc_buf.size = len;
+  msg.enc_buf.size = (unsigned)len;
  
   debug_print(&msg.enc_buf);      
 
@@ -2935,13 +2915,13 @@ int stun_keepalive(stun_handle_t *sh,
 		   tag_type_t tag, tag_value_t value,
 		   ...)
 {
-  int s = -1;
+  su_socket_t s = INVALID_SOCKET;
   unsigned int timeout = 0;
   ta_list ta;
   stun_discovery_t *sd;
   stun_request_t *req;
   stun_action_t action = stun_action_keepalive;
-  char ipaddr[SU_ADDRSIZE + 2] = { 0 };
+  char addr[SU_ADDRSIZE];
 
   enter;
 
@@ -2972,8 +2952,7 @@ int stun_keepalive(stun_handle_t *sh,
   req = stun_request_create(sd);
 
   SU_DEBUG_3(("%s: Starting to send STUN keepalives to %s:%u\n", __func__, 
-	      inet_ntop(sa->su_family, SU_ADDR(sa), 
-			ipaddr, sizeof(ipaddr)),
+	      inet_ntop(sa->su_family, SU_ADDR(sa), addr, sizeof(addr)),
 	      (unsigned) ntohs(sa->su_port)));
   
   if (stun_make_binding_req(sh, req, req->sr_msg, 0, 0) < 0 ||
@@ -3057,7 +3036,7 @@ int stun_keepalive_destroy(stun_handle_t *sh, su_socket_t s)
 
 int stun_process_request(su_socket_t s, stun_msg_t *req,
 			 int sid, su_sockaddr_t *from_addr,
-			 int from_len)
+			 socklen_t from_len)
 {
   stun_msg_t resp;
   su_sockaddr_t mod_addr[1] = {{ 0 }}, src_addr[1] = {{ 0 }}, chg_addr[1] = {{ 0 }};
