@@ -76,10 +76,10 @@ typedef uint64_t su_ntp_t;
 /** Represent NTP consttant */
 #define SU_NTP_C(x) SU_U64_C(x)
 
-#define SU_TIME_CMP(t1, t2) \
- (long)((t1.tv_sec - t2.tv_sec != 0) ? \
-   (t1.tv_sec - t2.tv_sec) : \
-     (t1.tv_usec - t2.tv_usec))
+#define SU_TIME_CMP(t1, t2) su_time_cmp(t1, t2)
+
+/** Seconds from 1.1.1900 to 1.1.1970. @NEW_1_12_4 */
+#define SU_TIME_EPOCH 2208988800UL 
 
 SOFIAPUBFUN su_time_t su_now(void);
 SOFIAPUBFUN void su_time(su_time_t *tv);
