@@ -94,15 +94,15 @@ static int li_scope4(uint32_t ip4);
 
 /** @brief Request local address information.
  *
- * The function su_getlocalinfo() gathers the network interfaces and the
- * addresses corresponding to them, checks if they match to the search
- * criteria specifed by @a hints and returns a list of matching local
- * address information in the @a res. The local address information may
- * include IPv4/IPv6 addresses, interface name, interface index, address
- * scope, and domain names corresponding to the local addresses.
+ * Gather the network interfaces and the addresses corresponding to them,
+ * check if they match to the search criteria specifed by @a hints and
+ * return a list of matching local address information in the @a
+ * return_localinfo. The local address information may include IPv4/IPv6
+ * addresses, interface name, interface index, address scope, and domain
+ * names corresponding to the local addresses.
  *
- * @param hints specifies selection criteria
- * @param return_localinfo   return list of local addresses
+ * @param[in] hints specifies selection criteria
+ * @param[out] return_localinfo   return list of local addresses
  *
  * @par Selection criteria - hints
  *
@@ -114,15 +114,17 @@ static int li_scope4(uint32_t ip4);
  * The @a hints->li_flags contain flags, which can be combined with bit-wise
  * or.  The currently defined flags are as follows:
  *
- * - @c LI_V4MAPPED: when returning IPv4 addresses, map them as IPv6
+ * - #LI_V4MAPPED: when returning IPv4 addresses, map them as IPv6
  *   addresses.  If this flag is specified, IPv4 addresses are returned even
  *   if @a hints->li_family is set to @c AF_INET6.
- * - @c LI_CANONNAME: return the domain name (DNS PTR) corresponding to the 
+ * - #LI_CANONNAME: return the domain name (DNS PTR) corresponding to the 
  *   local address in @a li_canonname.
- * - @c LI_NAMEREQD: Do not return addresses not in DNS. 
- * - @c LI_NUMERIC: instead of domain name, return the text presentation of
+ * - #LI_NAMEREQD: Do not return addresses not in DNS. 
+ * - #LI_NUMERIC: instead of domain name, return the text presentation of
  *   the addresss in @a li_canonname.
- * - @c LI_IFNAME: return the interface name in @a li_ifname.
+ * - #LI_DOWN: include interfaces and their addresses even if the interfaces
+ *   are down. New in @VERSION_1_12_2.
+ * - #LI_IFNAME: return the interface name in @a li_ifname.
  *
  * @par Selection by address family - hints->li_family
  *
@@ -145,27 +147,35 @@ static int li_scope4(uint32_t ip4);
  * @par Selection by address scope - hints->li_scope
  *
  * If the field @a hints->li_scope is nonzero, only the addresses with
- * matching scope are returned.  The address scopes can be combined with
- * bitwise or. For instance, setting @a hints->li_scope to @c
- * LI_SCOPE_GLOBAL | @c LI_SCOPE_SITE, both the @e global and @e site-local
- * addresses are returned.
+ * matching scope are returned. The different address scopes can be combined
+ * with bitwise or. They are defined as follows
+ * - #LI_SCOPE_HOST: host-local address, valid within host (::1, 127.0.0.1/8)
+ * - #LI_SCOPE_LINK: link-local address, valid within link 
+ *   (IP6 addresses with prefix fe80::/10, 
+ *    IP4 addresses in net 169.254.0.0/16).
+ * - #LI_SCOPE_SITE: site-local address, addresses valid within organization
+ *   (IPv6 addresses with prefix  fec::/10,
+ *    private IPv4 addresses in nets 10.0.0.0/8, 172.16.0.0/12, 
+ *    and 192.168.0.0/16 as defined in @RFC1918)
+ * - #LI_SCOPE_GLOBAL: global address.
  *
- * @par 
- * For IPv4, the loopback addresses (addresses in the net 127) are currently
- * considered @e host-local, other addresses are @e global.
+ * For instance, setting @a hints->li_scope to @c LI_SCOPE_GLOBAL | @c
+ * LI_SCOPE_SITE, both the @e global and @e site-local addresses are
+ * returned.
+ *
+ * @sa @RFC1918, @RFC4291, su_sockaddr_scope()
  *
  * @par Selection by domain name - hints->li_canonname
  *
  * If this field is non-null, the domain name (DNS PTR) corresponding to
  * local IP addresses should match to the name given in this field.
  *
- * @return 
- * The function su_getlocalinfo() returns zero when successful, or
- * negative error code when failed.
+ * @return Zero (#ELI_NOERROR) when successful, or negative error code when
+ * failed.
  * 
- * @par diagnostics
- * The function su_gli_strerror() returns a string describing the error code
- * returned by su_getlocalinfo().
+ * @par Diagnostics
+ * Use su_gli_strerror() in order to obtain a string describing the error
+ * code returned by su_getlocalinfo().
  *
  */
 int su_getlocalinfo(su_localinfo_t const *hints, 
@@ -243,7 +253,14 @@ int su_getlocalinfo(su_localinfo_t const *hints,
   return error;
 }
 
-/** Free local address information. */
+/** Free local address information. 
+ *
+ * Free a list of su_localinfo_t structures obtained with su_getlocalinfo()
+ * or su_copylocalinfo() along with socket addresses and strings associated
+ * with them.
+ *
+ * @sa su_getlocalinfo(), su_copylocalinfo(), #su_localinfo_t
+ */
 void su_freelocalinfo(su_localinfo_t *tbf)
 {
   su_localinfo_t *li;
@@ -285,7 +302,7 @@ char const *su_gli_strerror(int error)
  */
 su_localinfo_t *su_copylocalinfo(su_localinfo_t const *li0)
 {
-  int n;
+  size_t n;
   su_localinfo_t *li, *retval = NULL, **lli = &retval;
 
 # define SLEN(s) ((s) ? strlen(s) + 1 : 0)
@@ -427,7 +444,7 @@ int localinfo4(su_localinfo_t const *hints, su_localinfo_t **rresult)
 #if defined(__APPLE_CC__)
   {
     su_sockaddr_t *sa;
-    unsigned int salen = sizeof(*sa);
+    socklen_t salen = sizeof(*sa);
     int scope = 0, gni_flags = 0;
 
     li = calloc(1, sizeof(su_localinfo_t));
@@ -585,9 +602,10 @@ int localinfo4(su_localinfo_t const *hints, su_localinfo_t **rresult)
       error = ELI_SYSTEM;
       goto err;
     }
-    /* Do not include interfaces that are down */
-    if ((ifreq->ifr_flags & IFF_UP) == 0) {
-      SU_DEBUG_9(("su_localinfo: if %s with index %d is down\n", if_name, if_index));
+    /* Do not include interfaces that are down unless explicitly asked */
+    if ((ifreq->ifr_flags & IFF_UP) == 0 && (hints->li_flags & LI_DOWN) == 0) {
+      SU_DEBUG_9(("su_localinfo: if %s with index %d is down\n", 
+		  if_name, if_index));
       continue;
     }
 #else
@@ -696,11 +714,11 @@ int localinfo4(su_localinfo_t const *hints, su_localinfo_t **rresult)
   su_localinfo_t *li = NULL, **lli = &tbf;
   su_sockaddr_t *su;
 #if SU_HAVE_IN6
-  int su_sockaddr_size = 
+  socklen_t su_sockaddr_size = 
     (hints->li_flags & LI_V4MAPPED) ? sizeof(*su) : sizeof(struct sockaddr_in);
   flags = hints->li_flags & (LI_V4MAPPED|LI_CANONNAME|LI_NUMERIC|LI_IFNAME);
 #else
-  int su_sockaddr_size = sizeof(struct sockaddr_in);
+  socklen_t su_sockaddr_size = sizeof(struct sockaddr_in);
   flags = hints->li_flags & (LI_CANONNAME|LI_NUMERIC|LI_IFNAME);
 #endif
 
@@ -935,6 +953,10 @@ int localinfo6(su_localinfo_t const *hints, su_localinfo_t **rresult)
 	li = calloc(1, sizeof(su_localinfo_t));
 	sa = calloc(1, sizeof(su_sockaddr_t));
 
+	sa->su_family = AF_INET6;
+	if (inet_pton(AF_INET6, addr, &sa->su_sin6.sin6_addr) <= 0)
+	  goto err;
+	
 	s = su_socket(AF_INET6, SOCK_DGRAM, 0);
 	if (s == -1) {
 	  SU_DEBUG_1(("su_localinfo: su_socket failed: %s\n", 
@@ -1041,6 +1063,10 @@ int bsd_localinfo(su_localinfo_t const hints[1],
     char const *ifname = 0;
     size_t ifnamelen = 0;
 
+    /* no ip address from if that is down */
+    if ((ifa->ifa_flags & IFF_UP) == 0 && (hints->li_flags & LI_DOWN) == 0)
+      continue;
+
     su = (su_sockaddr_t *)ifa->ifa_addr;
 
     if (!su)
@@ -1321,7 +1347,7 @@ int localinfo0(su_localinfo_t const *hints, su_localinfo_t **rresult)
 #endif
     char buffer[2048];
   } b = {{ 1 }};
-  socklen_t salen = sizeof(b);
+  DWORD salen = sizeof(b);
   int i, error = -1;
 #if SU_HAVE_IN6
   int v4_mapped = (hints->li_flags & LI_V4MAPPED) != 0;
@@ -1332,7 +1358,7 @@ int localinfo0(su_localinfo_t const *hints, su_localinfo_t **rresult)
   *rresult = NULL;
 
   s = su_socket(family, SOCK_DGRAM, 0);
-  if (s == SOCKET_ERROR) {
+  if (s == INVALID_SOCKET) {
     SU_DEBUG_1(("su_getlocalinfo: %s: %s\n", "su_socket",
 		            su_strerror(su_errno())));
     return -1;
@@ -1493,7 +1519,7 @@ void li_sort(su_localinfo_t *i, su_localinfo_t **rresult)
 /**Get local IP address.
  *
  * @deprecated
- * Use su_localinfo() instead.
+ * Use su_getlocalinfo() instead.
  */
 int su_getlocalip(su_sockaddr_t *sa)
 {
